MODULE mo_be10 
  !-------------------------------------------------------------
  !
  !-------------------------------------------------------------
  USE mo_kind,               ONLY: wp, dp  
  USE mo_exception,          ONLY: finish
  USE mo_aero_mem,           ONLY: vmem3d
  USE mo_tracer,             ONLY: ntrac
  USE mo_decomposition,      ONLY: gl_dc => global_decomposition, &
       lcd=> local_decomposition

  !  USE netcdf

  IMPLICIT NONE  

  PRIVATE  

  PUBLIC:: init_be10, request_tracer_be10, ini_be10_memory, &
       be10_massbal, read_bcond_be10, read_bcond_be10_3d, chem_bcond_be10, &
       area_avg, wdep_be10, &
       be10_wdep_corr, str_trop_budgets, chem_bcond_pb210, read_cl36_bomb,     &
       read_cl36_bomb2, chem_cl36_bomb

  PUBLIC :: BE_FLAG, idt_be10, idt_be7, idt_cl36, idt_na22,  &
       idt_pb210,idt_rn222, be7_decay, burd_wdep, burd_wdep_be7,  &
       burd_wdep_cl36, burd_wdep_na22, burd_wdep_pb210,tropo_cl36,  &
       be10_ddep_acc, be7_ddep_acc, cl36_ddep_acc,pb210_ddep_acc, be10_sed_acc,   &
       be7_sed_acc, cl36_sed_acc, pb210_sed_acc, pb210conc, rn222_decay, &
       na22_sed_acc, na22_ddep_acc

   ! Be tracer index  
  INTEGER, SAVE         :: idt_be10, idt_be7, idt_na22, idt_be10030n,     &
       idt_be103060n,idt_be106090n,idt_be10030s,idt_be103060s,idt_be106090s,   &
       idt_be7030n,idt_be73060n,idt_be76090n,idt_be7030s,    &
       idt_be73060s, idt_be76090s, idt_cl36,idt_pb210,idt_rn222, &
       idt_be10st030n, idt_be10st030s, idt_be10st3060n, idt_be10st3060s,   &
       idt_be10st6090n, idt_be10st6090s, idt_be7st030n, idt_be7st030s,    &
       idt_be7st3060n,idt_be7st3060s, idt_be7st6090n,idt_be7st6090s, &
       idt_na22st030n, idt_na22st030s,    &
       idt_na22st3060n,idt_na22st3060s, idt_na22st6090n,idt_na22st6090s, &
       idt_na22030n, idt_na22030s,    &
       idt_na223060n,idt_na223060s, idt_na226090n,idt_na226090s



  !Flag for Beryllium deposition processes
  INTEGER, PARAMETER    :: BE_FLAG = 4

  INTEGER               :: bomb_idx, call_idx, bomb_lev 

  INTEGER, PARAMETER    :: neaw_trac=4 ! Number of EAWAG tracers (Be-10, Be-7, Cl-36)

  TYPE(vmem3d), PUBLIC,POINTER     :: tend_wdep(:)
  TYPE(vmem3d), PUBLIC,POINTER     :: tend_mass(:)
  TYPE(vmem3d), PUBLIC,POINTER     :: tend_scav(:)
 
  !uh:
  ! be10 input, production
  REAL(dp), POINTER     :: be10conc(:,:,:) 
  REAL(dp), POINTER     :: be10_prod_acc(:,:)
  REAL(dp), POINTER     :: be10_burden(:,:) 
  REAL(dp), POINTER     :: be10_wdep_acc(:,:) 
  REAL(dp), POINTER     :: be10_sed_acc(:,:) 
  REAL(dp), POINTER     :: be10_ddep_acc(:,:)
  REAL(dp), POINTER     :: probe10(:,:,:)
  REAL(dp), POINTER     :: burd_wdep(:,:,:)
  REAL(dp), POINTER     :: burd_wdep_be7(:,:,:)
  REAL(dp), POINTER     :: burd_wdep_cl36(:,:,:)
  REAL(dp), POINTER     :: burd_wdep_na22(:,:,:)
  REAL(dp), POINTER     :: burd_wdep_pb210(:,:,:)
  REAL(dp), POINTER, SAVE     :: tropo_cl36(:,:)
  REAL(dp), POINTER     :: test1(:,:,:)

  REAL(dp), POINTER     :: be7conc(:,:,:) 
  REAL(dp), POINTER     :: be7_prod_acc(:,:)
  REAL(dp), POINTER     :: be7_burden(:,:) 
  REAL(dp), POINTER     :: be7_decay(:,:,:)
  REAL(dp), POINTER     :: be7_dec_acc(:,:)
  REAL(dp), POINTER     :: be7_wdep_acc(:,:) 
  REAL(dp), POINTER     :: be7_sed_acc(:,:) 
  REAL(dp), POINTER     :: be7_ddep_acc(:,:) 
  REAL(dp), POINTER     :: be10pro_str(:,:)
  REAL(dp), POINTER     :: be10pro_tr(:,:)
  REAL(dp), POINTER     :: prostr(:,:,:)
  REAL(dp), POINTER     :: protr(:,:,:)
  REAL(dp), POINTER     :: be7pro_str(:,:)
  REAL(dp), POINTER     :: be7pro_tr(:,:)
  REAL(dp), POINTER     :: pro7str(:,:,:)
  REAL(dp), POINTER     :: pro7tr(:,:,:)

  REAL(dp), POINTER, SAVE     :: cl36conc(:,:,:) 
  REAL(dp), POINTER     :: cl36_prod_acc(:,:)
  REAL(dp), POINTER     :: cl36_burden(:,:) 
  REAL(dp), POINTER     :: cl36_wdep_acc(:,:) 
  REAL(dp), POINTER     :: cl36_sed_acc(:,:) 
  REAL(dp), POINTER     :: cl36_ddep_acc(:,:) 
  REAL(dp), POINTER     :: cl36pro_str(:,:)
  REAL(dp), POINTER     :: cl36pro_tr(:,:)
  REAL(dp), POINTER     :: pro36str(:,:,:)
  REAL(dp), POINTER     :: pro36tr(:,:,:)
  REAL(dp), POINTER, SAVE     :: cl36_bomb_input(:,:) 

  REAL(dp), POINTER, SAVE     :: na22conc(:,:,:) 
  REAL(dp), POINTER     :: na22_prod_acc(:,:)
  REAL(dp), POINTER     :: na22_burden(:,:) 
  REAL(dp), POINTER     :: na22_wdep_acc(:,:) 
  REAL(dp), POINTER     :: na22_sed_acc(:,:) 
  REAL(dp), POINTER     :: na22_ddep_acc(:,:) 
  REAL(dp), POINTER     :: na22pro_str(:,:)
  REAL(dp), POINTER     :: na22pro_tr(:,:)
  REAL(dp), POINTER     :: pro22str(:,:,:)
  REAL(dp), POINTER     :: pro22tr(:,:,:)

  REAL(dp), POINTER, SAVE     :: pb210conc(:,:,:) 
  REAL(dp), POINTER     :: rn222_prod(:,:) 
  REAL(dp), POINTER     :: pb210_prod_acc(:,:)
  REAL(dp), POINTER     :: pb210_burden(:,:) 
  REAL(dp), POINTER     :: pb210_wdep_acc(:,:) 
  REAL(dp), POINTER     :: pb210_sed_acc(:,:) 
  REAL(dp), POINTER     :: pb210_ddep_acc(:,:) 
  REAL(dp), POINTER     :: rn222_decay(:,:,:)
  REAL(dp), POINTER     :: rn222_dec_surf(:,:)
  REAL(dp), POINTER     :: rn222_burden(:,:)
  REAL(dp), POINTER     :: rn222_dec_acc(:,:) 


  REAL(dp), POINTER     :: burden_be10_str(:,:)
  REAL(dp), POINTER     :: burden_be10_tr(:,:)
  REAL(dp), POINTER     :: burden_be7_str(:,:)
  REAL(dp), POINTER     :: burden_be7_tr(:,:)
  REAL(dp), POINTER     :: burden_cl36_str(:,:)
  REAL(dp), POINTER     :: burden_cl36_tr(:,:)
  REAL(dp), POINTER     :: burden_na22_str(:,:)
  REAL(dp), POINTER     :: burden_na22_tr(:,:)
  REAL(dp), POINTER     :: burden_pb210_str(:,:)
  REAL(dp), POINTER     :: burden_pb210_tr(:,:)

  TYPE clbomb
     INTEGER :: byear(61)
     INTEGER :: bmonth(61)
     INTEGER :: bday(61)
     REAL(dp)    :: blat(61)
     REAL(dp)    :: blon(61)
     REAL(dp)    :: binput(61)
     INTEGER :: blevel(61)

  END TYPE clbomb

  TYPE(clbomb), SAVE :: clinput 



CONTAINS  

  SUBROUTINE init_be10   
    !----------------------------------------------------------    
    !
    !-----------------------------------------------------------   
    IMPLICIT NONE    

  END SUBROUTINE init_be10

  !---------------------------------------------------------------------------------------
  SUBROUTINE ini_be10_memory  
    !----------------------------------------------------------    
    ! 
    !-----------------------------------------------------------   
    USE mo_memory_base,       ONLY: t_stream, new_stream, add_stream_element, &         
         default_stream_setting, add_stream_reference
    USE mo_control,           ONLY: lbe10, lbe7, lcl36, lna22, lpb210, lbomb
    USE mo_tracer ,           ONLY: AEROSOLNUMBER, ntrac, trlist, ntrac
    USE mo_aero_m7,           ONLY: nmod
    USE mo_aero,              ONLY: nindex

    IMPLICIT NONE    

    TYPE(t_stream), POINTER   :: be10

    INTEGER                   :: jmod, jt

    ALLOCATE(tend_wdep(nmod))
    ALLOCATE(tend_mass(ntrac))
    ALLOCATE(tend_scav(ntrac))

    CALL new_stream(be10,'be10')
    CALL default_stream_setting(be10, units='1 kg-1 s-1',     &
         laccu=.FALSE., lrerun=.TRUE., lpost=.TRUE.)

    CALL add_stream_element(be10,'tropo_cl36',tropo_cl36,     &
         units='-',longname='tropopause index',lpost=.FALSE.,lrerun=.FALSE.)

    CALL add_stream_reference(be10,'gboxarea','geoloc', lpost=.TRUE.) 
    CALL add_stream_reference(be10,'DPG','aero', lpost=.TRUE.) 


    IF(lbe10) THEN 
       CALL add_stream_element(be10,'be10',be10conc,     &
            units='1 kg-1 s-1',longname='Production rate of Beryllium-10',lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'be10_prod_acc',be10_prod_acc,     &
            units='1 m-2 s-1',longname='Production of Beryllium-10', laccu=.TRUE.)

       CALL add_stream_element(be10,'be10_wdep_acc',be10_wdep_acc,     &
            units='1 m-2 s-1',longname='wet depos. of Be-10, acc', laccu=.TRUE.)

       CALL add_stream_element(be10,'be10_sed_acc',be10_sed_acc,     &
            units='1 m-2 s-1',longname='Sedimentation of Be-10, acc', laccu=.TRUE.)

       CALL add_stream_element(be10,'be10_ddep_acc',be10_ddep_acc,     &
            units='1 m-2 s-1',longname='Dry deposition of Be-10, acc', laccu=.TRUE.)


       CALL add_stream_element(be10,'burd_wdep',burd_wdep,     &
            units='1 m-2',longname='Massbal xxxx', lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'be10_burden',be10_burden,     &
            units='1 m-2',longname='Atmospheric burden of Beryllium-10',laccu=.TRUE.)

       CALL add_stream_element(be10,'burden_be10_str',burden_be10_str, &
            units='1 m-2', longname='Stratospheric burden of Beryllium-10',laccu=.TRUE.)

       CALL add_stream_element(be10,'burden_be10_tr',burden_be10_tr, &
            units='1 m-2', longname='Tropospheric burden of Beryllium-10',laccu=.TRUE.)

       CALL add_stream_element(be10,'probe10',probe10,units='',longname=''  &
            ,lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'prostr',prostr,units='',longname=''  &
            ,lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'protr',protr,units='',longname=''   &
            ,lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'be10pro_str',be10pro_str, &
            units='1 m-2 s-1', longname='Stratospheric production of Be-10',laccu=.TRUE.)

       CALL add_stream_element(be10,'be10pro_tr',be10pro_tr, &
            units='1 m-2 s-1', longname='Tropospheric production of Be-10',laccu=.TRUE.)

    END IF

    IF (lbe7) THEN
       CALL add_stream_element(be10,'pro7str',pro7str,units='',longname=''  &
            ,lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'pro7tr',pro7tr,units='',longname=''   &
            ,lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'be7pro_str',be7pro_str, &
            units='1 m-2 s-1', longname='Stratospheric production of Be-7',laccu=.TRUE.)

       CALL add_stream_element(be10,'be7pro_tr',be7pro_tr, &
            units='1 m-2 s-1', longname='Tropospheric production of Be-7',laccu=.TRUE.)

       CALL add_stream_element(be10,'be7',be7conc,     &
            units='1 kg-1 s-1',longname='Production rate of Beryllium-7',lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'be7_prod_acc',be7_prod_acc,     &
            units='1 m-2 s-1',longname='Production of Beryllium-7', laccu=.TRUE.)

       CALL add_stream_element(be10,'be7_decay',be7_decay,     &
            units='1 kg-1 s-1',longname='Radioactive decay of Beryllium-7')

       CALL add_stream_element(be10,'be7_dec_acc',be7_dec_acc,     &
            units='1 m-2 s-1',longname='Radioactive decay of Beryllium-7',laccu=.TRUE.)

       CALL add_stream_element(be10,'burd_wdep_be7',burd_wdep_be7,     &
            units='1 m-2 s-1',longname='Massbal xxxx', lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'be7_wdep_acc',be7_wdep_acc,     &
            units='1 m-2 s-1',longname='wet depos. of Beryllium-7', laccu=.TRUE.)

       CALL add_stream_element(be10,'be7_sed_acc',be7_sed_acc,     &
            units='1 m-2 s-1',longname='Sedimentation of Beryllium-7',laccu=.TRUE.)

       CALL add_stream_element(be10,'be7_ddep_acc',be7_ddep_acc,     &
            units='1 m-2 s-1',longname='Dry deposition of Beryllium-7',laccu=.TRUE.)

       CALL add_stream_element(be10,'be7_burden',be7_burden,     &
            units='1 m-2',longname='Atmospheric burden of Beryllium-7',laccu=.TRUE.)

       CALL add_stream_element(be10,'burden_be7_str',burden_be7_str, &
            units='1 m-2', longname='Stratospheric burden of Beryllium-7',laccu=.TRUE.)

       CALL add_stream_element(be10,'burden_be7_tr',burden_be7_tr, &
            units='1 m-2', longname='Tropospheric burden of Beryllium-7',laccu=.TRUE.)

    END IF

    IF (lcl36) THEN

       CALL add_stream_element(be10,'cl36',cl36conc,     &
            units='1 kg-1 s-1',longname='Production rate of Chlorine-36',lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'cl36_prod_acc',cl36_prod_acc,     &
            units='1 m-2 s-1',longname='Production of Chlorine-36',laccu=.TRUE.)

       CALL add_stream_element(be10,'cl36_burden',cl36_burden,     &
            units='1 m-2',longname='Atmospheric burden of Cl-36',laccu=.TRUE.)

       CALL add_stream_element(be10,'burden_cl36_str',burden_cl36_str, &
            units='1 m-2', longname='Stratospheric burden of Chlorine-36',laccu=.TRUE.)

       CALL add_stream_element(be10,'burden_cl36_tr',burden_cl36_tr, &
            units='1 m-2', longname='Tropospheric burden of Chlorine-36',laccu=.TRUE.)

       CALL add_stream_element(be10,'burd_wdep_cl36',burd_wdep_cl36,     &
            units='1 m-2',longname='Massbal xxxx', lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'cl36_wdep_acc',cl36_wdep_acc,     &
            units='1 m-2 s-1',longname='wet depos. of Chlorine-36',laccu=.TRUE.)

       CALL add_stream_element(be10,'cl36_sed_acc',cl36_sed_acc,     &
            units='1 m-2 s-1',longname='Sedimentation of Chlorine-36',laccu=.TRUE.)

       CALL add_stream_element(be10,'cl36_ddep_acc',cl36_ddep_acc,     &
            units='1 m-2 s-1',longname='Dry deposition of Chlorine-36',laccu=.TRUE.)

       CALL add_stream_element(be10,'pro36str',pro36str,units='',longname=''  &
            ,lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'pro36tr',pro36tr,units='',longname=''   &
            ,lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'cl36pro_str',cl36pro_str, &
            units='1 m-2 s-1', longname='Stratospheric production of Chlorine-36',laccu=.TRUE.)

       CALL add_stream_element(be10,'cl36pro_tr',cl36pro_tr, &
            units='1 m-2 s-1', longname='Tropospheric production of Chlorine-36',laccu=.TRUE.)

       IF (lbomb) THEN

          CALL add_stream_element(be10,'cl36_bomb_input',cl36_bomb_input,   &
               units='kg', longname='Chlorine-36 bomb input', &
               lpost=.FALSE., lrerun=.FALSE.)

       END IF

    END IF

    IF(lna22) THEN 
       CALL add_stream_element(be10,'na22',na22conc,     &
            units='1 kg-1 s-1',longname='Production rate of Sodium-22',lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'na22_prod_acc',na22_prod_acc,     &
            units='1 m-2 s-1',longname='Production of Sodium-22', laccu=.TRUE.)

       CALL add_stream_element(be10,'na22_wdep_acc',na22_wdep_acc,     &
            units='1 m-2 s-1',longname='wet depos. of Na-22, acc', laccu=.TRUE.)

       CALL add_stream_element(be10,'na22_sed_acc',na22_sed_acc,     &
            units='1 m-2 s-1',longname='Sedimentation of Na-22, acc', laccu=.TRUE.)

       CALL add_stream_element(be10,'na22_ddep_acc',na22_ddep_acc,     &
            units='1 m-2 s-1',longname='Dry deposition of Na-22, acc', laccu=.TRUE.)


       CALL add_stream_element(be10,'burd_wdep_na22',burd_wdep_na22,     &
            units='1 m-2',longname='Massbal xxxx', lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'na22_burden',na22_burden,     &
            units='1 m-2',longname='Atmospheric burden of Sodium-22',laccu=.TRUE.)

       CALL add_stream_element(be10,'burden_na22_str',burden_na22_str, &
            units='1 m-2', longname='Stratospheric burden of Sodium-22',laccu=.TRUE.)

       CALL add_stream_element(be10,'burden_na22_tr',burden_na22_tr, &
            units='1 m-2', longname='Tropospheric burden of Sodium-22',laccu=.TRUE.)

       CALL add_stream_element(be10,'pro22str',pro22str,units='',longname=''  &
            ,lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'pro22tr',pro22tr,units='',longname=''   &
            ,lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'na22pro_str',na22pro_str, &
            units='1 m-2 s-1', longname='Stratospheric production of Na-22',laccu=.TRUE.)

       CALL add_stream_element(be10,'na22pro_tr',na22pro_tr, &
            units='1 m-2 s-1', longname='Tropospheric production of Na-22',laccu=.TRUE.)

    END IF

    IF(lpb210) THEN 
       CALL add_stream_element(be10,'pb210',pb210conc,     &
            units='1 kg-1 s-1',longname='Production rate of Pb-210',lrerun=.FALSE.,lpost=.FALSE.)

       CALL add_stream_element(be10,'pb210_prod_acc',pb210_prod_acc,     &
            units='1 m-2 s-1',longname='Production of Lead-210', laccu=.TRUE.)

       CALL add_stream_element(be10,'rn222_prod',rn222_prod,     &
            units='1 m-2 s-1',longname='Production of Radon-222',lrerun=.FALSE.)

       CALL add_stream_element(be10,'pb210_wdep_acc',pb210_wdep_acc,     &
            units='1 m-2 s-1',longname='wet depos. of Lead-210, acc', laccu=.TRUE.)

       CALL add_stream_element(be10,'pb210_sed_acc',pb210_sed_acc,     &
            units='1 m-2 s-1',longname='Sedimentation of Lead-210, acc', laccu=.TRUE.)

       CALL add_stream_element(be10,'pb210_ddep_acc',pb210_ddep_acc,     &
            units='1 m-2 s-1',longname='Dry deposition of Lead-210, acc', laccu=.TRUE.)

       CALL add_stream_element(be10,'burd_wdep_pb210',burd_wdep_pb210,     &
            units='1 m-2',longname='Massbal xxxx', lpost=.FALSE.,lrerun=.FALSE.)

       CALL add_stream_element(be10,'pb210_burden',pb210_burden,     &
            units='1 m-2',longname='Atmospheric burden of Lead-210',laccu=.TRUE.)

       CALL add_stream_element(be10,'burden_pb210_str',burden_pb210_str, &
            units='1 m-2', longname='Stratospheric burden of Lead-210',laccu=.TRUE.)

       CALL add_stream_element(be10,'burden_pb210_tr',burden_pb210_tr, &
            units='1 m-2', longname='Tropospheric burden of Lead-10',laccu=.TRUE.)

       CALL add_stream_element(be10,'rn222_decay',rn222_decay,     &
            units='1 kg-1 s-1',longname='Radioactive decay of Rn-222',lrerun=.FALSE.,lpost=.FALSE.)

       CALL add_stream_element(be10,'rn222_dec_surf',rn222_dec_surf,     &
            units='1 m-2 s-1',longname='Radioactive decay of Rn-222',laccu=.TRUE.)

       CALL add_stream_element(be10,'rn222_burden',rn222_burden,     &
            units='1 m-2',longname='Atmospheric burden of Rn-222',laccu=.TRUE.)

       CALL add_stream_element(be10,'rn222_dec_acc',rn222_dec_acc,     &
            units='1 m-2 s-1',longname='Radioactive decay of Rn-222',laccu=.TRUE.)

    END IF


    DO jmod=1, nmod

       CALL add_stream_element(be10, 'TEND_WDEP_'//TRIM(trlist%ti(nindex(jmod))%fullname),   &
            tend_wdep(jmod)%ptr, units='1 kg-1 s-1',   &
            longname='wet deposition tendency', lpost=.FALSE.,lrerun=.FALSE. )!

    END DO

    DO jt=1,ntrac
       CALL add_stream_element(be10, 'TEND_MASS_'//TRIM(trlist%ti(jt)%fullname),   &
            tend_mass(jt)%ptr, units='1 kg-1 s-1',   &
            longname='wet deposition mass tendency', lpost=.FALSE.,lrerun=.FALSE. )

       CALL add_stream_element(be10, 'TEND_SCAV_'//TRIM(trlist%ti(jt)%fullname),   &
            tend_scav(jt)%ptr, units='-',   &
            longname='scav eff. per tracer',lrerun=.FALSE.,lpost=.FALSE.)


    END DO



  END SUBROUTINE ini_be10_memory
  !------------------------------------------------------------------------------------------
  !  SUBROUTINE chem2_be10

  !    IMPLICIT NONE


  !  END SUBROUTINE chem2_be10
  !----------------------------------------------------------------------------------
  SUBROUTINE request_tracer_be10
    !----------------------------------------------------------    
    !
    !-----------------------------------------------------------    

    USE mo_tracer,       ONLY: new_tracer,CONSTANT,INITIAL,ON, & 
         RESTART,AEROSOLNUMBER,GAS
    USE mo_exception,    ONLY: finish 
    USE mo_doctor,       ONLY: nout  
    USE mo_control,      ONLY: lbe10, lbe7, lcl36, lna22, lpb210, llat

    IMPLICIT NONE    

    INTEGER                 :: init, ierr

    init=RESTART+CONSTANT+INITIAL    


! GRIB CODES 301-> for COSMOGENIC RADIONUCLIDES

    IF (lbe10) THEN
       CALL new_tracer('be10','BE10', idx=idt_be10 ,units='1 kg-1', ierr=ierr, &  
            ninit=init, moleweight=10._dp, &
            ndrydep=BE_FLAG,&
            nphase=AEROSOLNUMBER, &
            nwetdep=BE_FLAG, &
            nsedi=BE_FLAG, &
            longname='Beryllium-10')

! If the latitudinal production distribution is not needed, comment out the 
! following new_tracer calls for Be-10

       IF (llat) THEN
          CALL new_tracer('be10st030n','BE10', idx=idt_be10st030n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 str 0-30N')

          CALL new_tracer('be10st030s','BE10', idx=idt_be10st030s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 str 0-30S')
          
          CALL new_tracer('be10st3060n','BE10', idx=idt_be10st3060n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 str 30-60N')
          
          CALL new_tracer('be10st3060s','BE10', idx=idt_be10st3060s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 str 30-60S')
          
          CALL new_tracer('be10st6090n','BE10', idx=idt_be10st6090n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 str 60-90N')
          
          CALL new_tracer('be10st6090s','BE10', idx=idt_be10st6090s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 str 60-90S')
          
          CALL new_tracer('be10030n','BE10', idx=idt_be10030n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 tropospheric 0-30 N')
          
          CALL new_tracer('be103060n','BE10', idx=idt_be103060n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 tr 30-60 N')
          
          CALL new_tracer('be106090n','BE10', idx=idt_be106090n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 tr 60-90 N')
          
          CALL new_tracer('be10030s','BE10', idx=idt_be10030s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 tr 0-30 S')
          
          CALL new_tracer('be103060s','BE10', idx=idt_be103060s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 tr 30-60 S')
          
          CALL new_tracer('be106090s','BE10', idx=idt_be106090s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=10._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Beryllium-10 tr 60-90 S')

       END IF

 !      IF (ierr /= 0 ) THEN       
 !         CALL finish ('request_tracer_be10',' failed to define tracer be10')    
 !      END IF
    END IF

    IF (lbe7) THEN
! For comments see Be-10 above
       CALL new_tracer('be7','BE10', idx=idt_be7 ,units='1 kg-1', ierr=ierr, &  
            ninit=init, moleweight=7._dp, &
            table=131   ,code=188,                        &
            ndrydep=BE_FLAG, &
            nphase=AEROSOLNUMBER, &
            nwetdep=BE_FLAG, &
            nsedi=BE_FLAG, &
            tdecay=6656248.7_dp, &
            longname='Beryllium-7')
       IF (llat) THEN
          CALL new_tracer('be7st030n','BE10', idx=idt_be7st030n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 str 0-30 N')
          
          CALL new_tracer('be7st030s','BE10', idx=idt_be7st030s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 str 0-30 S')
          
          CALL new_tracer('be7st3060n','BE10', idx=idt_be7st3060n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 str 30-60 N')
          
          CALL new_tracer('be7st3060s','BE10', idx=idt_be7st3060s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 str 30-60 S')
          
          CALL new_tracer('be7st6090n','BE10', idx=idt_be7st6090n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 str 60-90 N')
          
          CALL new_tracer('be7st6090s','BE10', idx=idt_be7st6090s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 str 60-90 S')
          


          CALL new_tracer('be7030n','BE10', idx=idt_be7030n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 tr 0-30 N')

          CALL new_tracer('be73060n','BE10', idx=idt_be73060n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 tr 30-60 N')

          CALL new_tracer('be76090n','BE10', idx=idt_be76090n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 tr 60-90 N')
          
          CALL new_tracer('be7030s','BE10', idx=idt_be7030s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 tr 0-30 S')

          CALL new_tracer('be73060s','BE10', idx=idt_be73060s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 tr 30-60 S')

          CALL new_tracer('be76090s','BE10', idx=idt_be76090s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=7._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               tdecay=6656248.7_dp, &
               longname='Beryllium-7 tr 60-90 S')

       END IF


       IF (ierr /= 0 ) THEN       
          CALL finish ('request_tracer_be10',' failed to define tracer be7')    
       END IF
    END IF

    IF (lcl36) THEN

       CALL new_tracer('cl36','BE10', idx=idt_cl36 ,units='1 kg-1', ierr=ierr, &  
            ninit=init, moleweight=36._dp,&
            ndrydep=BE_FLAG, &
            nphase=AEROSOLNUMBER, &
            nwetdep=BE_FLAG, &
            nsedi=BE_FLAG, &
            longname='Chlorine-36')

       IF (ierr /= 0 ) THEN       
          CALL finish ('request_tracer_be10',' failed to define tracer cl36')    
       END IF
    END IF

    IF (lna22) THEN
       CALL new_tracer('na22','BE10', idx=idt_na22 ,units='1 kg-1', ierr=ierr, &  
            ninit=init, moleweight=22._dp, &
            ndrydep=BE_FLAG,&
            nphase=AEROSOLNUMBER, &
            tdecay=118291760._dp, &
            nwetdep=BE_FLAG, &
            nsedi=BE_FLAG, &
            longname='Sodium-22')

       IF (llat) THEN
          CALL new_tracer('na22st030n','BE10', idx=idt_na22st030n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               tdecay=118291760._dp, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 str 0-30N')

          CALL new_tracer('na22st030s','BE10', idx=idt_na22st030s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               tdecay=118291760._dp, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 str 0-30S')
          
          CALL new_tracer('na22st3060n','BE10', idx=idt_na22st3060n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               tdecay=118291760._dp, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 str 30-60N')
          
          CALL new_tracer('na22st3060s','BE10', idx=idt_na22st3060s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               tdecay=118291760._dp, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 str 30-60S')
          
          CALL new_tracer('na22st6090n','BE10', idx=idt_na22st6090n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               tdecay=118291760._dp, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 str 60-90N')
          
          CALL new_tracer('na22st6090s','BE10', idx=idt_na22st6090s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               tdecay=118291760._dp, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 str 60-90S')
          
          CALL new_tracer('na22030n','BE10', idx=idt_na22030n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               tdecay=118291760._dp, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 tropospheric 0-30 N')
          
          CALL new_tracer('na223060n','BE10', idx=idt_na223060n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               tdecay=118291760._dp, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 tr 30-60 N')
          
          CALL new_tracer('na226090n','BE10', idx=idt_na226090n ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               nwetdep=BE_FLAG, &
               tdecay=118291760._dp, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 tr 60-90 N')
          
          CALL new_tracer('na22030s','BE10', idx=idt_na22030s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               tdecay=118291760._dp, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 tr 0-30 S')
          
          CALL new_tracer('na223060s','BE10', idx=idt_na223060s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               tdecay=118291760._dp, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 tr 30-60 S')
          
          CALL new_tracer('na226090s','BE10', idx=idt_na226090s ,units='1 kg-1', ierr=ierr, &  
               ninit=init, moleweight=22._dp, &
               ndrydep=BE_FLAG,&
               nphase=AEROSOLNUMBER, &
               tdecay=118291760._dp, &
               nwetdep=BE_FLAG, &
               nsedi=BE_FLAG, &
               longname='Sodium-22 tr 60-90 S')

       END IF

 !      IF (ierr /= 0 ) THEN       
 !         CALL finish ('request_tracer_be10',' failed to define tracer be10')    
 !      END IF
    END IF



    IF(lpb210) THEN
       CALL new_tracer('rn222','BE10', idx=idt_rn222 ,units='1 kg-1', ierr=ierr, &  
            ninit=init, moleweight=222._dp,&
            nphase=GAS, &
            tdecay=473665.6_dp,longname='Radon-222')
       IF (ierr /= 0 ) THEN       
          CALL finish ('request_tracer_be10',' failed to define tracer rn222')    
       END IF

       CALL new_tracer('pb210','BE10', idx=idt_pb210 ,units='1 kg-1', ierr=ierr, &  
            ninit=init, moleweight=210._dp,&
            ndrydep=BE_FLAG, &
            nphase=AEROSOLNUMBER, &
            nwetdep=BE_FLAG, &
            nsedi=BE_FLAG, &
            tdecay=1014579327._dp, &
            longname='Lead-210')

       IF (ierr /= 0 ) THEN       
          CALL finish ('request_tracer_be10',' failed to define tracer pb210')    
       END IF

    END IF


  END SUBROUTINE request_tracer_be10


  !------------------------------------------------------------------------------------------------- 

  SUBROUTINE be10_massbal(kproma,kbdim,klev,klevp1,krow,papp1,paphp1,pxtm1,pxtte)
!... Calculate the fluxes and burdens by integrating 
!... mixing ratios or tendencies over the atmospheric column 
!... Called from xt_burden

    USE mo_control,       ONLY: lbe10, lbe7, lcl36, lna22, lpb210 
    USE mo_constants,     ONLY: g
    USE mo_time_control,  ONLY: delta_time, time_step_len

    USE mo_tracer,        ONLY: ntrac
    USE mo_geoloc,        ONLY: gboxarea_2d


    IMPLICIT NONE

    INTEGER, INTENT(IN)       :: kbdim, klev, klevp1,kproma, krow

    REAL(dp), INTENT(IN)          :: paphp1(kbdim,klevp1),papp1(kbdim,klev), &
         pxtm1(kbdim,klev,ntrac),pxtte(kbdim,klev,ntrac)

    REAL(dp)        :: zdpg(kbdim,klev), dummy
    REAL(dp)        :: zxtp1_be10, zxtp1_cl36, zxtp1_be7
    REAL(dp)        :: zxtp1_pb210, zxtp1_rn222, zxtp1_na22

    INTEGER     :: jl, jk, j


    !... Initialisations:

    ! ... Calculate auxiliary variable dp/g:
    ! ... Uppermost level:

    DO jl=1,kproma
       zdpg(jl,1)=2._dp*(paphp1(jl,2)-papp1(jl,1))/g
    END DO

    !... Other levels:
    DO jk=2,klev
       DO jl=1,kproma
          zdpg(jl,jk)=(paphp1(jl,jk+1)-paphp1(jl,jk))/g
       END DO
    END DO

    !    production & wet deposition: sum over column

! Masarik & Beer, 1999 production rate of cosmogenic nuclides 
! increased by a factor of 1.5 
    DO jk=1,klev
       DO jl=1,kproma
          IF (lbe10) THEN
             be10_prod_acc(jl,krow)=be10_prod_acc(jl,krow)+     &
                  (be10conc(jl,jk,krow)*1.5_dp*zdpg(jl,jk)*delta_time)
             be10_wdep_acc(jl,krow)=be10_wdep_acc(jl,krow)+    &
                  (burd_wdep(jl,jk,krow)*zdpg(jl,jk)*delta_time)

          END IF
          IF (lbe7) THEN
             be7_prod_acc(jl,krow)=be7_prod_acc(jl,krow)+     &
                  (be7conc(jl,jk,krow)*1.5_dp*zdpg(jl,jk)*delta_time)
             be7_dec_acc(jl,krow)=be7_dec_acc(jl,krow)+     &
                  (be7_decay(jl,jk,krow)*zdpg(jl,jk)*delta_time)
             be7_wdep_acc(jl,krow)=be7_wdep_acc(jl,krow)+    &
                  (burd_wdep_be7(jl,jk,krow)*zdpg(jl,jk)*delta_time)
          END IF
          IF (lcl36) THEN
             cl36_prod_acc(jl,krow)=cl36_prod_acc(jl,krow)+     &
                  (cl36conc(jl,jk,krow)*1.5_dp*zdpg(jl,jk)*delta_time)
             cl36_wdep_acc(jl,krow)=cl36_wdep_acc(jl,krow)+    &
                  (burd_wdep_cl36(jl,jk,krow)*zdpg(jl,jk)*delta_time)
             
          END IF

          IF (lna22) THEN
             na22_prod_acc(jl,krow)=na22_prod_acc(jl,krow)+     &
                  (na22conc(jl,jk,krow)*1.5_dp*zdpg(jl,jk)*delta_time)
             na22_wdep_acc(jl,krow)=na22_wdep_acc(jl,krow)+    &
                  (burd_wdep_na22(jl,jk,krow)*zdpg(jl,jk)*delta_time)
             
          END IF

          IF (lpb210) THEN
             pb210_prod_acc(jl,krow)=pb210_prod_acc(jl,krow)+     &
                  (pb210conc(jl,jk,krow)*zdpg(jl,jk)*delta_time)
             pb210_wdep_acc(jl,krow)=pb210_wdep_acc(jl,krow)+    &
                  (burd_wdep_pb210(jl,jk,krow)*zdpg(jl,jk)*delta_time)
             rn222_dec_acc(jl,krow)=rn222_dec_acc(jl,krow)+     &
                  (rn222_decay(jl,jk,krow)*zdpg(jl,jk)*delta_time)
          END IF
       END DO
    END DO


    ! burden: sum over column  

    IF (lbe10) THEN

       DO jk=1,klev
          DO jl=1,kproma

             zxtp1_be10=pxtm1(jl,jk,idt_be10)+pxtte(jl,jk,idt_be10)*time_step_len
             be10_burden(jl,krow)=be10_burden(jl,krow)+(zxtp1_be10*zdpg(jl,jk))*delta_time
          END DO
       END DO
    END IF
    IF (lbe7) THEN
       DO jk=1,klev
          DO jl=1,kproma
             zxtp1_be7=pxtm1(jl,jk,idt_be7)+pxtte(jl,jk,idt_be7)*time_step_len
       
             be7_burden(jl,krow)=be7_burden(jl,krow)+(zxtp1_be7*zdpg(jl,jk))*delta_time
          END DO
       END DO
    END IF
    IF (lcl36) THEN
       DO jk=1,klev
          DO jl=1,kproma
             zxtp1_cl36=pxtm1(jl,jk,idt_cl36)+pxtte(jl,jk,idt_cl36)*time_step_len
             cl36_burden(jl,krow)=cl36_burden(jl,krow)+(zxtp1_cl36*zdpg(jl,jk))*delta_time
          END DO
       END DO

    END IF

    IF (lna22) THEN
       DO jk=1,klev
          DO jl=1,kproma
             zxtp1_na22=pxtm1(jl,jk,idt_na22)+pxtte(jl,jk,idt_na22)*time_step_len
             na22_burden(jl,krow)=na22_burden(jl,krow)+(zxtp1_na22*zdpg(jl,jk))*delta_time
          END DO
       END DO

    END IF

    IF (lpb210) THEN
      DO jk=1,klev
          DO jl=1,kproma
             zxtp1_pb210=pxtm1(jl,jk,idt_pb210)+pxtte(jl,jk,idt_pb210)*time_step_len
             pb210_burden(jl,krow)=pb210_burden(jl,krow)+(zxtp1_pb210*zdpg(jl,jk))*delta_time
             zxtp1_rn222=pxtm1(jl,jk,idt_rn222)+pxtte(jl,jk,idt_rn222)*time_step_len
             rn222_burden(jl,krow)=rn222_burden(jl,krow)+(zxtp1_rn222*zdpg(jl,jk))*delta_time
          END DO
       END DO
    END IF
 


  END SUBROUTINE be10_massbal
  !--------------------------------------------------------------------------------------------------  
  SUBROUTINE read_bcond_be10_3d(ithisyear,ithismonth)
! reads in the production rates of cosmogenic nuclides
! called from call_read_bcond in call_submodels
    USE mo_mpi,           ONLY: p_parallel_io
    USE mo_mpi,           ONLY: p_parallel, p_pe, p_io
!    USE mo_decomposition, ONLY: lcd => local_decomposition, &
!         gl_dc => global_decomposition
    USE mo_doctor,        ONLY: nout
    USE mo_transpose,     ONLY: scatter_gp
    USE mo_read_netcdf77, ONLY: read_var_nf77_3d

    USE mo_control,       ONLY: lbe10, lbe7, lcl36, lna22

    IMPLICIT NONE

    INTEGER, INTENT(IN)       :: ithismonth, ithisyear

    CHARACTER(LEN=256)        :: cname
    CHARACTER(LEN=8)          :: cyear, cmonth
    REAL(dp), POINTER             :: prod_be10(:,:,:)
    REAL(dp), POINTER             :: ztmp(:,:,:)

    INTEGER                   :: ierr, jlon, jlat, jlev, jeaw_trac(neaw_trac),j
    LOGICAL                   :: lex

    !uh:
    DO j=1,neaw_trac
       jeaw_trac(j)=0
    END DO
! How many tracers have to be read in?
    IF (lbe10) jeaw_trac(1)=1 
    IF (lbe7) jeaw_trac(2)=1 
    IF (lcl36) jeaw_trac(3)=1 
    IF (lna22) jeaw_trac(4)=1 

    write(cyear,fmt='(I4)') ithisyear
    write(cmonth,fmt='(I2)') ithismonth

    DO j=1,neaw_trac
       IF(jeaw_trac(j)<1) CYCLE
       IF (p_pe==p_io) THEN

          IF (j==1.AND.jeaw_trac(j)>0) THEN
             cname='input_be10_'//trim(adjustl(cyear))//'_'     &
                  //trim(adjustl(cmonth))//'.nc'   
          ELSEIF (j==2.AND.jeaw_trac(j)>0) THEN
             cname='input_be7_'//trim(adjustl(cyear))//'_'     &
                  //trim(adjustl(cmonth))//'.nc'   
          ELSEIF (j==3.AND.jeaw_trac(j)>0) THEN
             cname='input_cl36_'//trim(adjustl(cyear))//'_'     &
                  //trim(adjustl(cmonth))//'.nc'
          ELSEIF (j==4.AND.jeaw_trac(j)>0) THEN
             cname='input_na22_'//trim(adjustl(cyear))//'_'     &
                  //trim(adjustl(cmonth))//'.nc'
          ELSE 
             CYCLE
          END IF
          INQUIRE(file=trim(cname),exist=lex)
          IF (.NOT. lex) THEN
             WRITE(nout,*) '-----------------------------'
             WRITE(nout,*) '--- Missing file: -----------'
             WRITE(nout,*) '------', TRIM(cname)
             WRITE(nout,*) '-----------------------------'
          END IF

          ALLOCATE(prod_be10(lcd%nlon,lcd%nlat,lcd%nlev)) 
          ALLOCATE(ztmp(lcd%nlon,lcd%nlev,lcd%nlat))

          CALL read_var_nf77_3d(cname,'lon','lat','lev','prod_be10',  &
               prod_be10, ierr)
          
          ! change unit atoms/g -> atoms/kg
          DO jlev=1,lcd%nlev
             DO jlat=1,lcd%nlat
                DO jlon=1,lcd%nlon 
                   ztmp(jlon,jlev,jlat)=1000._dp*prod_be10(jlon,jlat,jlev)

                END DO
             END DO
          END DO
       END IF ! if p_pe==p_io

       IF (j==1.AND.jeaw_trac(j)>0) CALL scatter_gp(ztmp,be10conc,gl_dc)
       IF (j==2.AND.jeaw_trac(j)>0) CALL scatter_gp(ztmp,be7conc,gl_dc)
       IF (j==3.AND.jeaw_trac(j)>0) CALL scatter_gp(ztmp,cl36conc,gl_dc)
       IF (j==4.AND.jeaw_trac(j)>0) CALL scatter_gp(ztmp,na22conc,gl_dc)

       IF (p_parallel_io) THEN
          DEALLOCATE(prod_be10)
          DEALLOCATE(ztmp)
       END IF
    END DO  ! j=1,neaw_trac


  END SUBROUTINE read_bcond_be10_3d
  !--------------------------------------------------------------------------------------------

  SUBROUTINE read_bcond_be10(ithisyear,ithismonth)
! reads in the production rates of cosmogenic nuclides
! called from call_read_bcond in call_submodels
    USE mo_mpi,           ONLY: p_parallel_io
    USE mo_mpi,           ONLY: p_parallel, p_pe, p_io
!    USE mo_decomposition, ONLY: lcd => local_decomposition, &
!         gl_dc => global_decomposition
    USE mo_doctor,        ONLY: nout
    USE mo_transpose,     ONLY: scatter_gp
    USE mo_read_netcdf77, ONLY: read_var_nf77_2d

    USE mo_control,       ONLY: lbe10, lbe7, lcl36, lna22

    IMPLICIT NONE

    INTEGER, INTENT(IN)       :: ithismonth, ithisyear

    CHARACTER(LEN=256)        :: cname
    CHARACTER(LEN=8)          :: cyear, cmonth
    REAL(dp), POINTER             :: prod_be10(:,:)
    REAL(dp), POINTER             :: ztmp(:,:,:)

    INTEGER                   :: ierr, jlon, jlat, jlev, jeaw_trac(neaw_trac),j
    LOGICAL                   :: lex

    !uh:
    DO j=1,neaw_trac
       jeaw_trac(j)=0
    END DO
! How many tracers have to be read in?
    IF (lbe10) jeaw_trac(1)=1 
    IF (lbe7) jeaw_trac(2)=1 
    IF (lcl36) jeaw_trac(3)=1 
    IF (lna22) jeaw_trac(4)=1 

    write(cyear,fmt='(I4)') ithisyear
    write(cmonth,fmt='(I2)') ithismonth

    DO j=1,neaw_trac
       IF(jeaw_trac(j)<1) CYCLE
       IF (p_pe==p_io) THEN

          IF (j==1.AND.jeaw_trac(j)>0) THEN
             cname='input_be10_'//trim(adjustl(cyear))//'_'     &
                  //trim(adjustl(cmonth))//'.nc'   
          ELSEIF (j==2.AND.jeaw_trac(j)>0) THEN
             cname='input_be7_'//trim(adjustl(cyear))//'_'     &
                  //trim(adjustl(cmonth))//'.nc'   
          ELSEIF (j==3.AND.jeaw_trac(j)>0) THEN
             cname='input_cl36_'//trim(adjustl(cyear))//'_'     &
                  //trim(adjustl(cmonth))//'.nc'
          ELSEIF (j==4.AND.jeaw_trac(j)>0) THEN
             cname='input_na22_'//trim(adjustl(cyear))//'_'     &
                  //trim(adjustl(cmonth))//'.nc'
          ELSE 
             CYCLE
          END IF
          INQUIRE(file=trim(cname),exist=lex)
          IF (.NOT. lex) THEN
             WRITE(nout,*) '-----------------------------'
             WRITE(nout,*) '--- Missing file: -----------'
             WRITE(nout,*) '------', TRIM(cname)
             WRITE(nout,*) '-----------------------------'
          END IF

          ALLOCATE(prod_be10(lcd%nlat,lcd%nlev)) 
          ALLOCATE(ztmp(lcd%nlon,lcd%nlev,lcd%nlat))

          CALL read_var_nf77_2d(cname,'lat','lev','prod_be10',  &
               prod_be10, ierr)
          

          ! change unit atoms/g -> atoms/kg
          DO jlev=1,lcd%nlev
             DO jlat=1,lcd%nlat
                DO jlon=1,lcd%nlon 
                   ztmp(jlon,jlev,jlat)=1000._dp*prod_be10(jlat,jlev)

                END DO
             END DO
          END DO
       END IF ! if p_pe==p_io

       IF (j==1.AND.jeaw_trac(j)>0) CALL scatter_gp(ztmp,be10conc,gl_dc)
       IF (j==2.AND.jeaw_trac(j)>0) CALL scatter_gp(ztmp,be7conc,gl_dc)
       IF (j==3.AND.jeaw_trac(j)>0) CALL scatter_gp(ztmp,cl36conc,gl_dc)
       IF (j==4.AND.jeaw_trac(j)>0) CALL scatter_gp(ztmp,na22conc,gl_dc)

       IF (p_parallel_io) THEN
          DEALLOCATE(prod_be10)
          DEALLOCATE(ztmp)
       END IF
    END DO  ! j=1,neaw_trac


  END SUBROUTINE read_bcond_be10
  !--------------------------------------------------------------------------------------------
  SUBROUTINE chem_bcond_be10(kbdim,klev,kproma,krow,pxtte)

! ... save the read-in Be-10 production rates in total tendency arrays.
    USE mo_tracer,        ONLY: ntrac
    USE mo_control,       ONLY: lbe10, lbe7, lcl36, lna22, llat
    USE mo_constants,     ONLY: g
    USE mo_geoloc,        ONLY: philat_2d
    USE mo_time_control,  ONLY: lstart
    USE mo_mpi,           ONLY: p_parallel_io

    IMPLICIT NONE

    INTEGER, INTENT(IN)       :: kbdim, klev,kproma, krow

    REAL(dp), INTENT(inout)       :: pxtte(kbdim,klev,ntrac)

    INTEGER :: jl, jk, ktropo


! Masarik & Beer, 1999 production rates of cosmogenic radionuclides rather low ->
! increased by a factor of 1.5. See Heikkila et. al, ACP, 2008 for discussion.
    IF (lbe10) THEN
       DO jk=1,klev
          DO jl=1,kproma
             pxtte(jl,jk,idt_be10)=pxtte(jl,jk,idt_be10)     &
                  +be10conc(jl,jk,krow)*1.5_dp
             ktropo=int(tropo_cl36(jl,krow))
             IF (jk <= ktropo) THEN
! Stratospheric production
                prostr(jl,jk,krow)= be10conc(jl,jk,krow)*1.5_dp
! Divide the input into 30 degree latitude bands
                IF (llat) THEN
                   IF (philat_2d(jl,krow) > 60._dp) THEN
                      pxtte(jl,jk,idt_be10st6090n)=pxtte(jl,jk,idt_be10st6090n) &
                           +be10conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 30._dp .AND. philat_2d(jl,krow)<=60._dp) THEN
                      pxtte(jl,jk,idt_be10st3060n)=pxtte(jl,jk,idt_be10st3060n)     &
                           +be10conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 0._dp .AND. philat_2d(jl,krow)<=30._dp) THEN
                      pxtte(jl,jk,idt_be10st030n)=pxtte(jl,jk,idt_be10st030n)     &
                           +be10conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -30._dp .AND. philat_2d(jl,krow)<=0._dp) THEN
                      pxtte(jl,jk,idt_be10st030s)=pxtte(jl,jk,idt_be10st030s)     &
                           +be10conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -60._dp .AND. philat_2d(jl,krow)<=-30._dp) THEN
                      pxtte(jl,jk,idt_be10st3060s)=pxtte(jl,jk,idt_be10st3060s)     &
                           +be10conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow)<=-60._dp) THEN
                      pxtte(jl,jk,idt_be10st6090s)=pxtte(jl,jk,idt_be10st6090s)     &
                           +be10conc(jl,jk,krow)*1.5_dp
                   END IF
                END IF

             ELSE ! jk> ktropo
! Tropospheric production
                protr(jl,jk,krow)= be10conc(jl,jk,krow)*1.5_dp
! Divide the input into 30 degree latitude bands
                IF (llat) THEN
                   IF (philat_2d(jl,krow) > 60._dp) THEN
                      pxtte(jl,jk,idt_be106090n)=pxtte(jl,jk,idt_be106090n) &
                           +be10conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 30._dp .AND. philat_2d(jl,krow)<=60._dp) THEN
                      pxtte(jl,jk,idt_be103060n)=pxtte(jl,jk,idt_be103060n)     &
                           +be10conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 0._dp .AND. philat_2d(jl,krow)<=30._dp) THEN
                      pxtte(jl,jk,idt_be10030n)=pxtte(jl,jk,idt_be10030n)     &
                           +be10conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -30._dp .AND. philat_2d(jl,krow)<=0._dp) THEN
                      pxtte(jl,jk,idt_be10030s)=pxtte(jl,jk,idt_be10030s)     &
                           +be10conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -60._dp .AND. philat_2d(jl,krow)<=-30._dp) THEN
                      pxtte(jl,jk,idt_be103060s)=pxtte(jl,jk,idt_be103060s)     &
                           +be10conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow)<=-60._dp) THEN
                      pxtte(jl,jk,idt_be106090s)=pxtte(jl,jk,idt_be106090s)     &
                           +be10conc(jl,jk,krow)*1.5_dp
                   END IF

                END IF

             END IF


          END DO
       END DO

    END IF
     

    IF (lbe7) THEN
! See comments for Be-10 above

       DO jk=1,klev
          DO jl=1,kproma
             pxtte(jl,jk,idt_be7)=pxtte(jl,jk,idt_be7)     &
                  +be7conc(jl,jk,krow)*1.5_dp
             ktropo=int(tropo_cl36(jl,krow))
             IF (jk <= ktropo) THEN
                pro7str(jl,jk,krow)= be7conc(jl,jk,krow)*1.5_dp
                
                IF (llat) THEN
                   IF (philat_2d(jl,krow) > 60._dp) THEN
                      pxtte(jl,jk,idt_be7st6090n)=pxtte(jl,jk,idt_be7st6090n) &
                           +be7conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 30._dp .AND. philat_2d(jl,krow)<=60._dp) THEN
                      pxtte(jl,jk,idt_be7st3060n)=pxtte(jl,jk,idt_be7st3060n)     &
                           +be7conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 0._dp .AND. philat_2d(jl,krow)<=30._dp) THEN
                      pxtte(jl,jk,idt_be7st030n)=pxtte(jl,jk,idt_be7st030n)     &
                           +be7conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -30._dp .AND. philat_2d(jl,krow)<=0._dp) THEN
                      pxtte(jl,jk,idt_be7st030s)=pxtte(jl,jk,idt_be7st030s)     &
                           +be7conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -60._dp .AND. philat_2d(jl,krow)<=-30._dp) THEN
                      pxtte(jl,jk,idt_be7st3060s)=pxtte(jl,jk,idt_be7st3060s)     &
                           +be7conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow)<=-60._dp) THEN
                      pxtte(jl,jk,idt_be7st6090s)=pxtte(jl,jk,idt_be7st6090s)     &
                           +be7conc(jl,jk,krow)*1.5_dp
                   END IF

                END IF
             ELSE
                pro7tr(jl,jk,krow)= be7conc(jl,jk,krow)*1.5_dp

                IF (llat) THEN
                   IF (philat_2d(jl,krow) > 60._dp) THEN
                      pxtte(jl,jk,idt_be76090n)=pxtte(jl,jk,idt_be76090n) &
                           +be7conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 30._dp .AND. philat_2d(jl,krow)<=60._dp) THEN
                      pxtte(jl,jk,idt_be73060n)=pxtte(jl,jk,idt_be73060n)     &
                           +be7conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 0._dp .AND. philat_2d(jl,krow)<=30._dp) THEN
                      pxtte(jl,jk,idt_be7030n)=pxtte(jl,jk,idt_be7030n)     &
                           +be7conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -30._dp .AND. philat_2d(jl,krow)<=0._dp) THEN
                      pxtte(jl,jk,idt_be7030s)=pxtte(jl,jk,idt_be7030s)     &
                           +be7conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -60._dp .AND. philat_2d(jl,krow)<=-30._dp) THEN
                      pxtte(jl,jk,idt_be73060s)=pxtte(jl,jk,idt_be73060s)     &
                           +be7conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow)<=-60._dp) THEN
                      pxtte(jl,jk,idt_be76090s)=pxtte(jl,jk,idt_be76090s)     &
                           +be7conc(jl,jk,krow)*1.5_dp
                   END IF

                END IF
             END IF


          END DO
       END DO

     END IF

    IF (lcl36) THEN
       DO jk=1,klev 
          DO jl=1,kproma
             pxtte(jl,jk,idt_cl36)=pxtte(jl,jk,idt_cl36)     &
                  +cl36conc(jl,jk,krow)*1.5_dp
             ktropo=int(tropo_cl36(jl,krow))
             IF (jk <= ktropo) THEN
! Stratospherc production
                pro36str(jl,jk,krow)= cl36conc(jl,jk,krow)*1.5_dp

             ELSE
! Tropospheric production
                pro36tr(jl,jk,krow)= cl36conc(jl,jk,krow)*1.5_dp
             END IF
          END DO
       END DO

    END IF

    IF (lna22) THEN
       DO jk=1,klev
          DO jl=1,kproma
             pxtte(jl,jk,idt_na22)=pxtte(jl,jk,idt_na22)     &
                  +na22conc(jl,jk,krow)*1.5_dp
             ktropo=int(tropo_cl36(jl,krow))
             IF (jk <= ktropo) THEN
! Stratospheric production
!                write(*,*) klev,kproma,na22conc(jl,jk,krow)
!                write(*,*) pro22str(jl,jk,krow),'klev,kproma,naconc'
                pro22str(jl,jk,krow)= na22conc(jl,jk,krow)*1.5_dp

! Divide the input into 30 degree latitude bands
                IF (llat) THEN
                   
                   IF (philat_2d(jl,krow) > 60._dp) THEN
                      pxtte(jl,jk,idt_na22st6090n)=pxtte(jl,jk,idt_na22st6090n) &
                           +na22conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 30._dp .AND. philat_2d(jl,krow)<=60._dp) THEN
                      pxtte(jl,jk,idt_na22st3060n)=pxtte(jl,jk,idt_na22st3060n)     &
                           +na22conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 0._dp .AND. philat_2d(jl,krow)<=30._dp) THEN
                      pxtte(jl,jk,idt_na22st030n)=pxtte(jl,jk,idt_na22st030n)     &
                           +na22conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -30._dp .AND. philat_2d(jl,krow)<=0._dp) THEN
                      pxtte(jl,jk,idt_na22st030s)=pxtte(jl,jk,idt_na22st030s)     &
                           +na22conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -60._dp .AND. philat_2d(jl,krow)<=-30._dp) THEN
                      pxtte(jl,jk,idt_na22st3060s)=pxtte(jl,jk,idt_na22st3060s)     &
                           +na22conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow)<=-60._dp) THEN
                      pxtte(jl,jk,idt_na22st6090s)=pxtte(jl,jk,idt_na22st6090s)     &
                           +na22conc(jl,jk,krow)*1.5_dp
                   END IF
                END IF

             ELSE ! jk> ktropo
! Tropospheric production
                pro22tr(jl,jk,krow)= na22conc(jl,jk,krow)*1.5_dp
! Divide the input into 30 degree latitude bands
                IF (llat) THEN
                   IF (philat_2d(jl,krow) > 60._dp) THEN
                      pxtte(jl,jk,idt_na226090n)=pxtte(jl,jk,idt_na226090n) &
                           +na22conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 30._dp .AND. philat_2d(jl,krow)<=60._dp) THEN
                      pxtte(jl,jk,idt_na223060n)=pxtte(jl,jk,idt_na223060n)     &
                           +na22conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > 0._dp .AND. philat_2d(jl,krow)<=30._dp) THEN
                      pxtte(jl,jk,idt_na22030n)=pxtte(jl,jk,idt_na22030n)     &
                           +na22conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -30._dp .AND. philat_2d(jl,krow)<=0._dp) THEN
                      pxtte(jl,jk,idt_na22030s)=pxtte(jl,jk,idt_na22030s)     &
                           +na22conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow) > -60._dp .AND. philat_2d(jl,krow)<=-30._dp) THEN
                      pxtte(jl,jk,idt_na223060s)=pxtte(jl,jk,idt_na223060s)     &
                           +na22conc(jl,jk,krow)*1.5_dp
                   ELSEIF (philat_2d(jl,krow)<=-60._dp) THEN
                      pxtte(jl,jk,idt_na226090s)=pxtte(jl,jk,idt_na226090s)     &
                           +na22conc(jl,jk,krow)*1.5_dp
                   END IF

                END IF

             END IF


          END DO
       END DO

    END IF


  END SUBROUTINE chem_bcond_be10

  !--------------------------------------------------------------------------------------------

  SUBROUTINE area_avg(kproma, klev, kbdim,krow, dep_idx, pxtm1, pvar_avgd, idt)
    !... Subroutine to area average the deposition velocity of all NUM tracers
    !... over modes to estimate the amount of Be-10, which is deposited by 
    !... the process in question (sedimentation or dry deposition).   

    !... Called from xt_sedimentation and xt_drydep

    USE mo_tracer,         ONLY: ntrac, trlist
    USE mo_aero_m7,        ONLY: cmr2ras, nmod
    USE mo_aero,           ONLY: nindex
    USE mo_aero_mem,       ONLY: rwet, vdrydep
    USE mo_constants,      ONLY: api
    USE mo_doctor,         ONLY: nout
    USE mo_control,        ONLY: lbe10,lbe7,lcl36,lna22,lpb210
    USE mo_time_control,   ONLY: time_step_len

    IMPLICIT NONE

    INTEGER,INTENT(IN)    :: kproma, kbdim, klev, krow

    INTEGER,INTENT(IN)    :: dep_idx, idt ! deposition index
    !  1 = sedimentation, 2 = dry deposition
    ! 3 = wet deposition
    REAL(dp)                  :: zvt_be10(kbdim,klev), zvt_dry(kbdim) 
    ! area averaged sedimentation vel./ dry dep velocity 
    REAL(dp), POINTER         :: v_p(:,:) ! dry dep velocity pointer

    REAL(dp), INTENT(IN)      :: pxtm1(kbdim,klev,ntrac)

    REAL(dp), INTENT(INOUT)   :: pvar_avgd(kbdim,klev,ntrac)!variable to be averaged 
    ! not needed for dry deposition

    INTEGER               :: jt, jk, jl, jmod, imod, ktropo

    REAL(dp)                  :: ztotarea(kbdim,klev), &
         ! total surface area of all particles in all modes  
    zarea(kbdim,klev,nmod), zareamod(kbdim,nmod), &
         zarealev(kbdim), zdeparea(kbdim,klev),  &
         zvt_be10_old(kbdim,klev)
    ! average area of a particle in a mode 
    REAL(dp), POINTER         :: rwet_p(:,:,:)
    REAL(dp), POINTER         :: tend_avg_p(:,:,:)


    INTEGER, POINTER      :: ndep

    ! Initialisations

    DO jk=1,klev
       DO jl=1,kproma
          ztotarea(jl,jk) = 0._dp
          zvt_be10(jl,jk) = 0._dp
          zdeparea(jl,jk) = 0._dp

       END DO
    END DO

    DO jmod=1,nmod
       DO jk=1,klev
          DO jl=1,kproma
             zarea(jl,jk,jmod) = 0._dp
          END DO
       END DO
    END DO

    DO jl=1,kproma
       zvt_dry(jl)=0._dp
       zarealev(jl)=0._dp
    END DO

    DO jmod=1,nmod
       DO jl=1,kproma
          zareamod(jl,jmod)=0._dp
       END DO
    END DO

    !... Calculate total surface area of all particles in all modes (using rwet)

    DO jt=1,ntrac
       SELECT CASE (dep_idx)
       CASE (1) 
          ndep => trlist%ti(jt)%nsedi
       CASE (2)
          ndep => trlist%ti(jt)%ndrydep
       CASE (3)
          ndep => trlist%ti(jt)%nwetdep
       END SELECT

       IF (ndep > 0.AND.ndep < BE_FLAG) THEN 
          imod=trlist%ti(jt)%mode
          ! Only modes which are AEROSOLNUMBER
          IF (imod>0) THEN
             IF (jt==nindex(imod)) THEN
                rwet_p      => rwet(imod)%ptr

                DO jk=1,klev
                   DO jl=1,kproma
                      zarea(jl,jk,imod)=4._dp*api*     &
                           (cmr2ras(imod)*rwet_p(jl,jk,krow))**2

                      ztotarea(jl,jk)=ztotarea(jl,jk)+   &
                           (zarea(jl,jk,imod)*pxtm1(jl,jk,jt))              
                      IF (dep_idx==2) THEN
                         zareamod(jl,imod)=zareamod(jl,imod)+    &
                              (zarea(jl,jk,imod)*pxtm1(jl,jk,jt))
                         zarealev(jl)=zarealev(jl)+     &
                              (zarea(jl,jk,imod)*pxtm1(jl,jk,jt))
                      END IF
                   END DO
                END DO
                
                DO jk=1,klev
                   DO jl=1,kproma
                      ktropo=int(tropo_cl36(jl,krow))
                      IF (jk >=ktropo+1) THEN
                         IF (abs(pvar_avgd(jl,jk,jt))*time_step_len<pxtm1(jl,jk,jt)) THEN
                            IF(dep_idx==3) THEN
                               zdeparea(jl,jk)=zdeparea(jl,jk)+     &
                                    MIN(0._dp,zarea(jl,jk,imod)*pvar_avgd(jl,jk,jt)*time_step_len)
                            END IF
                         END IF
                      END IF ! jk=ktropo+1, klev
                   END DO ! jl=1,kproma
                END DO
             END IF
             
          END IF
       END IF

    END DO !jt=1,ntrac

    DO jt=1,ntrac
       SELECT CASE (dep_idx)
       CASE (1) 
          ndep => trlist%ti(jt)%nsedi
       CASE (2)
          ndep => trlist%ti(jt)%ndrydep
          v_p => vdrydep(jt)%ptr
!       CASE (3)
!          ndep => trlist%ti(jt)%nwetdep
       END SELECT

       IF (ndep > 0.AND.ndep < BE_FLAG) THEN 

          imod=trlist%ti(jt)%mode
          !... Only modes which are AEROSOLNUMBER
          IF (imod > 0) THEN
             IF (jt==nindex(imod)) THEN 
                IF (dep_idx==1) THEN
                   DO jk=1,klev
                      DO jl=1,kproma
                         zvt_be10(jl,jk)=zvt_be10(jl,jk)+    &
                              (pvar_avgd(jl,jk,jt)*zarea(jl,jk,imod)   &
                              *pxtm1(jl,jk,jt))
                      END DO
                   END DO
                END IF
                IF (dep_idx==2) THEN
                   DO jl=1,kproma
                      zvt_dry(jl)=zvt_dry(jl)+    &
                           (v_p(jl,krow)*zareamod(jl,imod))
                   END DO
                END IF
             END IF !jt==nindex...
          END IF !(imod>0)
          CYCLE
       END IF
    END DO


    SELECT CASE(dep_idx)
    CASE(1)

       DO jk=1,klev
          DO jl=1,kproma
             IF (ztotarea(jl,jk)>epsilon(1.0_dp)) THEN 
                zvt_be10(jl,jk)=    &
                     zvt_be10(jl,jk)/ztotarea(jl,jk)
             ELSE 
                zvt_be10(jl,jk)=0._dp
             END IF
          END DO
       END DO
    CASE(2)
       DO jl=1,kproma
          IF(zarealev(jl)>epsilon(1.0_dp)) THEN
             zvt_dry(jl)=zvt_dry(jl)/zarealev(jl)
          ELSE
             zvt_dry(jl)=0._dp
          END IF
       END DO
!    CASE(3)
!       DO jl=1,kproma
!          ktropo=int(tropo_cl36(jl,krow))
!          DO jk=ktropo+1,klev
!             IF (ztotarea(jl,jk)>epsilon(1.0)) THEN 
!                !zdeparea < 0.
!                zvt_be10(jl,jk)=MIN(0.,zdeparea(jl,jk)*pxtm1(jl,jk,idt))/ztotarea(jl,jk)
!             ELSE 
!                zvt_be10(jl,jk)=0.
!             END IF!

!          END DO
!       END DO

    END SELECT

    !... Save the deposition velocity for Be-10.
    SELECT CASE (dep_idx)
    CASE(1)
       DO jk=1,klev
          DO jl=1,kproma
             pvar_avgd(jl,jk,idt)=zvt_be10(jl,jk)
          END DO
       END DO

    CASE(2)
       v_p => vdrydep(idt)%ptr
       DO jl=1,kproma
          v_p(jl,krow)=zvt_dry(jl)
       END DO
!    CASE (3)
!       DO jl=1,kproma
!          ktropo=int(tropo_cl36(jl,krow))
!          DO jk=ktropo+1,klev!!

!             IF (pxtm1(jl,jk,idt)>epsilon(1.0)) THEN!
!
!                pvar_avgd(jl,jk,idt)=zvt_be10(jl,jk)/time_step_len
!                IF (ABS(pvar_avgd(jl,jk,idt))*time_step_len>pxtm1(jl,jk,idt)) THEN
!
!                   pvar_avgd(jl,jk,idt)=-pxtm1(jl,jk,idt)/time_step_len
!                END IF!
!
!             ELSE
!                pvar_avgd(jl,jk,idt)=0.
!             END IF
!          END DO
!       END DO

    END SELECT

  END SUBROUTINE area_avg
  !--------------------------------------------------------------------------------------------

  SUBROUTINE wdep_be10(kproma, klev, kbdim, krow, dep_idx, pxtm1, pvar_avgd, idt)
! Calculate the wet deposition tendency for cosmogenic radionuclides and Pb-210
! Called from xt_wetdep
    USE mo_tracer,         ONLY: ntrac, trlist, AEROSOLMASS
    USE mo_aero_m7,        ONLY: cmr2ras, nmod
    USE mo_aero,           ONLY: nindex
    USE mo_aero_mem,       ONLY: rwet, vdrydep
    USE mo_constants,      ONLY: api
    USE mo_doctor,         ONLY: nout
    USE mo_control,        ONLY: lbe10,lbe7,lcl36,lna22,lpb210
    USE mo_time_control,   ONLY: time_step_len

    IMPLICIT NONE

    INTEGER,INTENT(IN)    :: kproma, kbdim, klev, krow

    INTEGER,INTENT(IN)    :: dep_idx, idt ! deposition index
    !  1 = sedimentation, 2 = dry deposition
    ! 3 = wet deposition
    REAL(dp)                  :: zvt_be10(kbdim,klev)

    REAL(dp), INTENT(IN)      :: pxtm1(kbdim,klev,ntrac)

    REAL(dp), INTENT(INOUT)   :: pvar_avgd(kbdim,klev,ntrac)!variable to be averaged 
 
    INTEGER               :: jt, jk, jl, jmod, imod, ktropo

    REAL(dp)                  :: zvt_tot(kbdim,klev), zxtratio(kbdim,klev)
 
    REAL(dp), POINTER         :: tend_scav_p(:,:,:)


    INTEGER, POINTER      :: ndep

    ! Initialisations
  

    DO jk=1,klev
       DO jl=1,kproma

          zvt_be10(jl,jk) = 0._dp
          zvt_tot(jl,jk) = 0._dp
          zxtratio(jl,jk) = 0._dp

       END DO
    END DO


    DO jt=1,ntrac
       ndep => trlist%ti(jt)%nwetdep

       IF (ndep > 0.AND.ndep < BE_FLAG) THEN 
          ! Only modes which are AEROSOLMASS
          ! only SULFATE for Be-10, Be-7, all for Pb-210

          IF (trlist%ti(jt)%nphase==AEROSOLMASS) THEN

             IF(lpb210.AND.idt==idt_pb210) THEN
                do jk = 1, klev
                   DO jl=1,kproma
                      ktropo=int(tropo_cl36(jl,krow))
                      if ( jk > ktropo ) then
                         IF (abs(pvar_avgd(jl,jk,jt))*time_step_len<pxtm1(jl,jk,jt)) THEN
                            zvt_be10(jl,jk)=zvt_be10(jl,jk)+pvar_avgd(jl,jk,jt)*time_step_len
                            zvt_tot(jl,jk)=zvt_tot(jl,jk)+pxtm1(jl,jk,jt)
                         END IF
                      endif
                   END DO
                END DO
             ELSE
                IF (trlist%ti(jt)%basename=='SO4') THEN

                   do jk = 1, klev
                      DO jl=1,kproma
                         ktropo=int(tropo_cl36(jl,krow))
                         if ( jk > ktropo ) then

                            IF (abs(pvar_avgd(jl,jk,jt))*time_step_len<pxtm1(jl,jk,jt)) THEN
                               zvt_be10(jl,jk)=zvt_be10(jl,jk)+pvar_avgd(jl,jk,jt)*time_step_len
                            END IF
                         endif
                      END DO
                   END DO

                ENDIF ! if SO4

                do jk = 1, klev
                   DO jl=1,kproma
                      ktropo=int(tropo_cl36(jl,krow))
                      if ( jk > ktropo ) then
                        IF (abs(pvar_avgd(jl,jk,jt))*time_step_len<pxtm1(jl,jk,jt)) THEN
                            zvt_tot(jl,jk)=zvt_tot(jl,jk)+pxtm1(jl,jk,jt)
                         END IF
                      endif
                   END DO
                END DO
             ENDIF ! if idt==idt_pb210
             !             END IF
          END IF
       END IF

    END DO !jt=1,ntrac

    DO jk=1,klev
       DO jl=1,kproma
          ktropo=int(tropo_cl36(jl,krow))
          if ( jk > ktropo ) then
             IF (zvt_tot(jl,jk)>epsilon(1.0_dp)) THEN
                zxtratio(jl,jk)=MIN(0._dp,zvt_be10(jl,jk)/zvt_tot(jl,jk))
             ELSE
                zxtratio(jl,jk)=0._dp
             END IF
          endif

       END DO
    END DO

    do jk = 1, klev
       DO jl=1,kproma
          ktropo=int(tropo_cl36(jl,krow))
          if ( jk > ktropo ) then 
             IF (pxtm1(jl,jk,idt)>epsilon(1.0_dp)) THEN
                pvar_avgd(jl,jk,idt)=zxtratio(jl,jk)*pxtm1(jl,jk,idt)/time_step_len
             ELSE
                pvar_avgd(jl,jk,idt)=0._dp
                
             END IF
          endif
       END DO
    END DO

  END SUBROUTINE wdep_be10


  !---------------------------------------------------------------------------------------------

  SUBROUTINE be10_wdep_corr(kproma,kbdim,klev,krow,pxttefix)
    !... correct the mass fixed convective wet deposition tendencies
    ! called from xt_massfix
    USE mo_tracer,        ONLY: trlist, AEROSOLMASS, AEROSOLNUMBER, ntrac
    USE mo_aero,          ONLY: nindex

    IMPLICIT NONE

    INTEGER, INTENT(IN) :: kproma,kbdim,klev, krow
    REAL(dp), INTENT(IN)    :: pxttefix(kbdim,klev,ntrac)
    REAL(dp), POINTER       :: tend_mass_p(:,:,:)
    REAL(dp), POINTER       :: tend_wdep_p(:,:,:)

    INTEGER :: jl, jk, jt, jmod

    !... update the mass-fixed wet deposition tendency:
    DO jt=1,ntrac
       IF(trlist%ti(jt)%nphase==AEROSOLMASS.AND.trlist%ti(jt)%nwetdep>0) THEN
          tend_mass_p => tend_mass(jt)%ptr
          DO jk=1,klev
             DO jl=1,kproma
                tend_mass_p(jl,jk,krow)=tend_mass_p(jl,jk,krow)+pxttefix(jl,jk,jt)

             END DO
          END DO
       ELSEIF(trlist%ti(jt)%nphase==AEROSOLNUMBER.AND.    &
            trlist%ti(jt)%nwetdep>0.AND.trlist%ti(jt)%nwetdep < BE_FLAG) THEN
          jmod=trlist%ti(jt)%mode
          tend_wdep_p => tend_wdep(jmod)%ptr
          DO jk=1,klev
             DO jl=1,kproma
                tend_wdep_p(jl,jk,krow)=tend_wdep_p(jl,jk,krow)+pxttefix(jl,jk,jt)
             END DO
          END DO
       END IF
    END DO


  END SUBROUTINE be10_wdep_corr


  !---------------------------------------------------------------------------------------------

  SUBROUTINE str_trop_budgets(kproma, kbdim, klev,krow, pxtm1, pxtte, pdpg)
! Calculate the burdens separately in the stratosphere and the troposphere
! Called from xt_burden

    USE mo_control,          ONLY: lbe10, lbe7, lcl36, lna22, lpb210
    USE mo_tracer,           ONLY: ntrac
    USE mo_time_control,     ONLY: time_step_len, delta_time
    USE mo_mpi,              ONLY: p_parallel_io


    IMPLICIT NONE

    INTEGER, INTENT(in) :: kproma,     kbdim,       klev, krow
    REAL(dp),    INTENT(in) :: pxtm1(kbdim,klev,ntrac), pxtte(kbdim,klev,ntrac)
    REAL(dp),    INTENT(in) :: pdpg(kbdim,klev)


    !--- Local Variables:

    INTEGER             :: jl, jk

    INTEGER             :: ktropo

    REAL(dp)                :: zxtp1


    IF (lbe10) THEN
       DO jk=1,klev
          DO jl=1,kproma
             ktropo=int(tropo_cl36(jl,krow))             
             IF (jk <= ktropo) THEN
                zxtp1=(pxtm1(jl,jk,idt_be10)+pxtte(jl,jk,idt_be10)*time_step_len)
                burden_be10_str(jl,krow)=burden_be10_str(jl,krow)+zxtp1*pdpg(jl,jk)*delta_time
                be10pro_str(jl,krow)=be10pro_str(jl,krow)+prostr(jl,jk,krow)*pdpg(jl,jk)*delta_time
             END IF
          END DO
       END DO
    END IF
    IF (lbe7) THEN
       DO jk=1,klev
          DO jl=1,kproma
             ktropo=int(tropo_cl36(jl,krow))             
             IF (jk <= ktropo) THEN
                zxtp1=(pxtm1(jl,jk,idt_be7)+pxtte(jl,jk,idt_be7)*time_step_len)
                burden_be7_str(jl,krow)=burden_be7_str(jl,krow)+zxtp1*pdpg(jl,jk)*delta_time
                be7pro_str(jl,krow)=be7pro_str(jl,krow)+pro7str(jl,jk,krow)*pdpg(jl,jk)*delta_time
             END IF
          END DO
       END DO
    END IF
    IF (lcl36) THEN 
       DO jk=1,klev
          DO jl=1,kproma
             ktropo=int(tropo_cl36(jl,krow))             
             IF (jk <= ktropo) THEN
                zxtp1=(pxtm1(jl,jk,idt_cl36)+pxtte(jl,jk,idt_cl36)*time_step_len)
                burden_cl36_str(jl,krow)=burden_cl36_str(jl,krow)+zxtp1*pdpg(jl,jk)*delta_time

                cl36pro_str(jl,krow)=cl36pro_str(jl,krow)+pro36str(jl,jk,krow)*pdpg(jl,jk)*delta_time
             END IF
          END DO
       END DO
    END IF

    IF (lna22) THEN 
       DO jk=1,klev
          DO jl=1,kproma
             ktropo=int(tropo_cl36(jl,krow))             
             IF (jk <= ktropo) THEN
                zxtp1=(pxtm1(jl,jk,idt_na22)+pxtte(jl,jk,idt_na22)*time_step_len)
                burden_na22_str(jl,krow)=burden_na22_str(jl,krow)+zxtp1*pdpg(jl,jk)*delta_time

                na22pro_str(jl,krow)=na22pro_str(jl,krow)+pro22str(jl,jk,krow)*pdpg(jl,jk)*delta_time
             END IF
          END DO
       END DO
    END IF


    IF (lpb210) THEN
       DO jk=1,klev
          DO jl=1,kproma
             ktropo=int(tropo_cl36(jl,krow))             
             IF (jk <= ktropo) THEN
                zxtp1=(pxtm1(jl,jk,idt_pb210)+pxtte(jl,jk,idt_pb210)*time_step_len)
                burden_pb210_str(jl,krow)=burden_pb210_str(jl,krow)+zxtp1*pdpg(jl,jk)*delta_time
             END IF
          END DO
       END DO
    END IF

! troposphere
    IF (lbe10) THEN
       DO jk=1,klev
          DO jl=1,kproma
             ktropo=int(tropo_cl36(jl,krow))             
             IF (jk > ktropo) THEN
                zxtp1=(pxtm1(jl,jk,idt_be10)+pxtte(jl,jk,idt_be10)*time_step_len)

                burden_be10_tr(jl,krow)=burden_be10_tr(jl,krow)+zxtp1*pdpg(jl,jk)*delta_time
                be10pro_tr(jl,krow)=be10pro_tr(jl,krow)+protr(jl,jk,krow)*pdpg(jl,jk)*delta_time
             END IF
          END DO
       END DO
    END IF
    IF (lbe7)  tHEN
       DO jk=1,klev
          DO jl=1,kproma
             ktropo=int(tropo_cl36(jl,krow))             
             IF (jk > ktropo) THEN
                zxtp1=(pxtm1(jl,jk,idt_be7)+pxtte(jl,jk,idt_be7)*time_step_len)
                burden_be7_tr(jl,krow)=burden_be7_tr(jl,krow)+zxtp1*pdpg(jl,jk)*delta_time
                be7pro_tr(jl,krow)=be7pro_tr(jl,krow)+pro7tr(jl,jk,krow)*pdpg(jl,jk)*delta_time
             END IF
          END DO
       END DO
    END IF
    IF (lcl36) THEN
       DO jk=1,klev
          DO jl=1,kproma
             ktropo=int(tropo_cl36(jl,krow))             
             IF (jk > ktropo) THEN
                zxtp1=(pxtm1(jl,jk,idt_cl36)+pxtte(jl,jk,idt_cl36)*time_step_len)
                burden_cl36_tr(jl,krow)=burden_cl36_tr(jl,krow)+zxtp1*pdpg(jl,jk)*delta_time

                cl36pro_tr(jl,krow)=cl36pro_tr(jl,krow)+pro36tr(jl,jk,krow)*pdpg(jl,jk)*delta_time
             END IF
          END DO
       END DO
    END IF
    IF (lna22) THEN
       DO jk=1,klev
          DO jl=1,kproma
             ktropo=int(tropo_cl36(jl,krow))             
             IF (jk > ktropo) THEN
                zxtp1=(pxtm1(jl,jk,idt_na22)+pxtte(jl,jk,idt_na22)*time_step_len)
                burden_na22_tr(jl,krow)=burden_na22_tr(jl,krow)+zxtp1*pdpg(jl,jk)*delta_time

                na22pro_tr(jl,krow)=na22pro_tr(jl,krow)+pro22tr(jl,jk,krow)*pdpg(jl,jk)*delta_time
             END IF
          END DO
       END DO
    END IF

    IF (lpb210) THEN
       DO jk=1,klev
          DO jl=1,kproma
             ktropo=int(tropo_cl36(jl,krow))             
             IF (jk > ktropo) THEN
                zxtp1=(pxtm1(jl,jk,idt_pb210)+pxtte(jl,jk,idt_pb210)*time_step_len)

                burden_pb210_tr(jl,krow)=burden_pb210_tr(jl,krow)+zxtp1*pdpg(jl,jk)*delta_time
             END IF
          END DO
       END DO
    END IF

  END SUBROUTINE str_trop_budgets

  !---------
  !------------------------------------------------------------------------------------

  SUBROUTINE chem_bcond_pb210(kbdim,klev,klevp1,kproma,krow,pxtte,paphp1,loland,loglac,ptslm1)
! Calculate the production rate of Pb-210 which is the same as the radioactive
! decay rate of Rn-222 (calculated in mo_sub_echam)
! Called from call_chem_bcond in call_submodels
    USE mo_tracer,         ONLY: ntrac
    USE mo_constants,      ONLY: g

    IMPLICIT NONE

    INTEGER, INTENT(in)    :: kproma, kbdim, klev, klevp1, krow

    LOGICAL, INTENT(in)    :: loland(kbdim)

    LOGICAL, INTENT(in)    :: loglac(kbdim)

    REAL(dp), INTENT(in)       :: ptslm1(kbdim), paphp1(kbdim,klevp1)

    REAL(dp),    INTENT(inout) :: pxtte(kbdim,klev,ntrac)

    INTEGER                :: jl

    REAL(dp)                 :: zgdp


    !... assume constant Rn222 emissions from land 1.0 cm-2 s-1 -> 
    !... atmospheric concentrations in kg-1 s-1 in the lowest model level
    !... = emissions *g/dp
    DO jl=1,kproma
       zgdp=g/(paphp1(jl,klevp1)-paphp1(jl,klev))

       IF (loland(jl).AND..NOT.loglac(jl)) THEN
          !... lower emissions by a factor of 3 under freezing conditions  
          IF (ptslm1(jl) < 273.15_dp) THEN
             pxtte(jl,klev,idt_rn222)=pxtte(jl,klev,idt_rn222)+zgdp*1.E4_dp/3._dp
             rn222_prod(jl,krow)=1.E4_dp/3._dp
          ELSE
             pxtte(jl,klev,idt_rn222)=pxtte(jl,klev,idt_rn222)+zgdp*1.E4_dp
             rn222_prod(jl,krow)=1.E4_dp
          END IF
       END IF
    END DO


  END SUBROUTINE chem_bcond_pb210


  !------------------------------------------------------------------------------------


  !------------------------------------------------------------------------------------

  SUBROUTINE read_cl36_bomb
!... Subroutine to be used only for the bomb test peak of Cl-36 (1952-1974)
!... Called from call_read_bcond in call_submodels during the bomb years
!... This routine reads in the bomb input from the file "clinput.dat"   

    USE mo_mpi,           ONLY: p_pe, p_io, p_bcast, p_parallel_io
    USE mo_filename,      ONLY: find_next_free_unit    
    USE mo_doctor,        ONLY: nout  

    IMPLICIT NONE


    INTEGER             :: iunit, ifile1, ifile2, i, ierr, j
    LOGICAL             :: lex
    
! File dimensions
    ifile1=8
    ifile2=61

    IF (p_parallel_io) iunit = find_next_free_unit (20, 99)

    IF (p_pe == p_io) THEN
       INQUIRE (file='clinput.dat',exist=lex) !if IO processor, check file existence
    END IF
     
    CALL p_bcast(lex,p_io)
     
    IF (p_pe == p_io) THEN
       IF (lex) THEN

          OPEN (iunit,file='clinput.dat',form='FORMATTED')
          REWIND(iunit)
       ELSE
          WRITE(nout,*) '----------------------------------------'
          WRITE(nout,*) '       File clinput.dat not found'
          WRITE(nout,*) '----------------------------------------'
          WRITE(nout,*) '----------------------------------------'
          WRITE(nout,*) '----------------------------------------'

       END IF
    END IF

    IF (lex) THEN
       IF (p_pe == p_io) THEN
 
          DO
             READ(iunit,*,IOSTAT=ierr)
             
             IF(ierr < 0 ) THEN
                REWIND(iunit)
                EXIT
             ELSE IF(ierr > 0) THEN
                CALL finish('read_cl36_bomb','Error reading input file clinput.dat (2)')
             END IF


          END DO

          !--- Read the Cl-36 input:

          DO i=1,ifile2
             READ(iunit,*,IOSTAT=ierr) clinput%byear(i), clinput%bmonth(i),   &
                  clinput%bday(i), clinput%blat(i), clinput%blon(i),     &
                  clinput%binput(i), clinput%blevel(i)
             IF(ierr < 0 ) THEN
                EXIT
             ELSE IF(ierr > 0) THEN
                CALL finish('read_cl36_bomb','Error reading input file clinput.dat (3)')
             END IF
                
             IF(clinput%blon(i)<0._dp) clinput%blon(i)=clinput%blon(i)+360._dp
             
          END DO
          
!          CALL p_bcast(clinput%binput,p_io)

       END IF
    END IF


 
    IF (p_pe == p_io) THEN

       CLOSE (iunit)    

    END IF

  END SUBROUTINE read_cl36_bomb


!!---------------------------------------------------------------------------------------------------------!


   SUBROUTINE read_cl36_bomb2(ithisday, ithismonth, ithisyear)
! This routine finds the grid box indexes of the C-36 bomb input 
! Called from call_read_bcond in call_submodels


     USE mo_transpose,     ONLY: scatter_gp
     USE mo_control,       ONLY: nlon, ngl,lbomb
!     USE mo_decomposition, ONLY: gl_dc => global_decomposition
     USE mo_tracer,        ONLY: ntrac
     USE mo_aero_tools,    ONLY: geoindex
     USE mo_mpi,           ONLY: p_bcast,p_io, p_pe, p_parallel_io, &
                                 p_parallel

     IMPLICIT NONE
!
     INTEGER :: ithismonth, ithisyear, ithisday, idim1, idim2, i, klon,klat

     REAL(dp), POINTER          :: ztmp(:,:)
     LOGICAL                :: lfound

!..... Initialisations

     idim1=8
     idim2=61

     lfound=.FALSE.

! let the i/o processor do the read-in

     IF (p_parallel_io) THEN

!!! uh

     i=1
     call_idx=1
 !    CALL p_bcast(call_idx,p_pe)
     ALLOCATE (ztmp(nlon,ngl)) 
     DO WHILE (i<=idim2)
        ztmp=0._dp

       IF (ithisyear==clinput%byear(i)) THEN
           IF (ithismonth==clinput%bmonth(i).AND.   &
                ithisday==clinput%bday(i)) THEN
 
              !... Find the indices of the respective latitudes and longitudes
              CALL geoindex(clinput%blon(i),clinput%blat(i),klon,klat)
! divide the mass between 9 grid points to avoid extreme gradients between the
! neighbouring grid boxes
              
              ztmp(klon-1,klat-1) = clinput%binput(i)/9._dp
              ztmp(klon  ,klat-1) = clinput%binput(i)/9._dp
              ztmp(klon+1,klat-1) = clinput%binput(i)/9._dp
              ztmp(klon-1,klat  ) = clinput%binput(i)/9._dp
              ztmp(klon  ,klat  ) = clinput%binput(i)/9._dp
              ztmp(klon+1,klat  ) = clinput%binput(i)/9._dp
              ztmp(klon-1,klat+1) = clinput%binput(i)/9._dp
              ztmp(klon  ,klat+1) = clinput%binput(i)/9._dp
              ztmp(klon+1,klat+1) = clinput%binput(i)/9._dp

!            CALL scatter_gp(ztmp, cl36_bomb_input, gl_dc)
 
              lbomb=.TRUE. ! (TRUE for a bomb day)

              lfound=.TRUE. ! (TRUE if a bomb was found)
              bomb_lev=clinput%blevel(i)
!              CALL p_bcast(lbomb, p_pe)
!              CALL p_bcast(bomb_idx, p_pe)

              EXIT
           END IF
           i=i+1
           IF (clinput%byear(i)>ithisyear) EXIT
        ELSE
           i=i+1
        END IF
     END DO


     end if ! p_pe=p_io

     CALL p_bcast(lfound,p_io)

     IF(lfound) THEN
        CALL scatter_gp(ztmp, cl36_bomb_input, gl_dc)

 !       IF (p_parallel) THEN
        CALL p_bcast(lbomb,p_io)
        CALL p_bcast(bomb_lev,p_io)
 !       END IF

     END IF

     IF (p_pe==p_io) THEN
        DEALLOCATE(ztmp)
     END IF !p_pe==p_io

  

   END SUBROUTINE read_cl36_bomb2


!----------------------------------------------------------


   SUBROUTINE chem_cl36_bomb(pxtte, paphp1,kbdim,kproma, klev, klevp1,krow)
! add the read-in bomb input to the mixing ratio tendencies of Cl-36 in atoms/kg
! of the correct vertical level (divided between 4 vertical levels)
! The reading in is done in 2 time steps to avoid the oscillation and mass loss
! caused by the leap-frog time integration scheme.
! Called from call_chem_bcond in call_submodels


     USE mo_control,       ONLY: lbomb
     USE mo_constants,     ONLY: g, avo  
     USE mo_time_control,  ONLY: time_step_len
     USE mo_geoloc,        ONLY: gboxarea_2d
!     USE mo_decomposition, ONLY: lcd => local_decomposition 
     USE mo_mpi,           ONLY: p_bcast, p_pe, p_io, p_parallel
     
     IMPLICIT NONE
     
     INTEGER :: indlev, indlevp1, indlevm1, indlevp2
     INTEGER :: jl, jk
     
     INTEGER, INTENT(in) :: kbdim, kproma, klev, klevp1, krow
     REAL(dp), INTENT(inout) :: pxtte(kbdim,klev,ntrac)
     REAL(dp), INTENT(in)    :: paphp1(kbdim,klevp1)
     REAL(dp)                :: zdpg(kbdim,klev), dummyburden, zxt
     
!..... Initialisations

! Calculate auxiliary variable DP/G 

     DO jk=2,klev
        DO jl=1,kproma
           zdpg(jl,jk)=(paphp1(jl,jk+1)-paphp1(jl,jk))/g
        END DO
     END DO

     indlev=bomb_lev
     indlevp1=indlev+1
     indlevp2=indlev+2
     indlevm1=indlev-1


     DO jl=1,kproma
        
        pxtte(jl,indlev,idt_cl36)=     &
             pxtte(jl,indlev,idt_cl36)+    &
! convert kg into atoms/kg/s and divide input between four levels
! read in during 2 time steps - leap-frog problem
             cl36_bomb_input(jl,krow)/4._dp*1000._dp*avo/    &
             (zdpg(jl,indlev)*gboxarea_2d(jl,krow)*36._dp*time_step_len*4._dp)

        pxtte(jl,indlevp1,idt_cl36)=     &
             pxtte(jl,indlevp1,idt_cl36)+    &

! input divided by 4. because the gloabl input was about 80 kg, see
! Heikkila et al., Comparison of the Cl-36 bomb peak modelled using the 
! ECHAM5-HAM.... etc., ACP, submitted.
             cl36_bomb_input(jl,krow)/4._dp*1000.*avo/    &
             (zdpg(jl,indlevp1)*gboxarea_2d(jl,krow)*36._dp*time_step_len*4._dp)

        pxtte(jl,indlevp2,idt_cl36)=     &
             pxtte(jl,indlevp2,idt_cl36)+    &
             cl36_bomb_input(jl,krow)/4._dp*1000._dp*avo/    &
             (zdpg(jl,indlevp2)*gboxarea_2d(jl,krow)*36._dp*time_step_len*4._dp)

        pxtte(jl,indlevm1,idt_cl36)=     &
             pxtte(jl,indlevm1,idt_cl36)+    &
             cl36_bomb_input(jl,krow)/4._dp*1000._dp*avo/    &
             (zdpg(jl,indlevm1)*gboxarea_2d(jl,krow)*36._dp*time_step_len*4._dp)
     END DO

! Check if this is the first or second call of the routine
     IF (krow==lcd%ngpblks.AND.call_idx<3) THEN
        call_idx=call_idx+1

     END IF

     IF (krow==lcd%ngpblks.AND.call_idx==3) THEN
        lbomb=.FALSE.
        call_idx=1

     END IF


   END SUBROUTINE chem_cl36_bomb



!------------------------------------------------------------------------------------


  !---------
END MODULE mo_be10




!---------
