SUBROUTINE cuflx(    kproma, kbdim, klev, klevp1,                      &
           pqen,     pqsen,    ptenh,    pqenh,                        &
           ktrac,                                                      &
!---Included for scavenging in xtwetdep (Philip Stier, 28/03/01):-------
           krow,                                                       &
           pxtte,    pxtu,     ptu,                                    &
           pmwc,     pmrateprecip,                                     &
!---End Included for scavenging-----------------------------------------
           pxtenh,   pmfuxt,   pmfdxt,                                 &
           paphp1,   pgeoh,                                            &
           kcbot,    kctop,    kdtop,                                  &
           ktype,    lddraf,   ldcum,                                  &
           pmfu,     pmfd,     pmfus,    pmfds,                        &
           pmfuq,    pmfdq,    pmful,                                  &
           pdmfup,   pdmfdp,   prfl,     prain,                        &
           pcpcu,                                                      &
           pten,     psfl,     pdpmel,   ktopm2)
!
!          M.TIEDTKE         E.C.M.W.F.     7/86 MODIF. 12/89
!
!          PURPOSE
!          -------
!
!          THIS ROUTINE DOES THE FINAL CALCULATION OF CONVECTIVE
!          FLUXES IN THE CLOUD LAYER AND IN THE SUBCLOUD LAYER
!
!          INTERFACE
!          ---------
!          THIS ROUTINE IS CALLED FROM *CUMASTR*.
!
!          EXTERNALS
!          ---------
!          NONE
!
USE mo_kind,         ONLY: dp
USE mo_constants,    ONLY: g, alf, cpd, tmelt, vtmpc2
USE mo_physc2,       ONLY: cevapcu
USE mo_time_control, ONLY: time_step_len
#ifdef __ibm__
USE mo_specfun,      ONLY: merge
#endif

!---Included for scavenging in xtwetdep (Philip Stier, 28/03/01):-------
USE mo_constants,    ONLY : rd
USE mo_tracer,       ONLY : trlist
USE mo_aero_trac,    ONLY : idt_so2,   idt_so4,   idt_ms4ns, idt_ms4ks, &
                            idt_ms4as, idt_ms4cs
!---End Included for scavenging-----------------------------------------
!
IMPLICIT NONE
!
INTEGER, INTENT (IN) :: kproma, kbdim, klev, klevp1, ktrac
INTEGER, INTENT (OUT):: ktopm2
!
REAL(dp):: pqen(kbdim,klev),        pqsen(kbdim,klev),                 &
           ptenh(kbdim,klev),       pqenh(kbdim,klev),                 &
           paphp1(kbdim,klevp1),    pgeoh(kbdim,klev)
!
REAL(dp):: pmfu(kbdim,klev),        pmfd(kbdim,klev),                  &
           pmfus(kbdim,klev),       pmfds(kbdim,klev),                 &
           pmfuq(kbdim,klev),       pmfdq(kbdim,klev),                 &
           pdmfup(kbdim,klev),      pdmfdp(kbdim,klev),                &
           pmful(kbdim,klev),                                          &
           prfl(kbdim),             prain(kbdim)
REAL(dp):: pten(kbdim,klev),        pdpmel(kbdim,klev),                &
           psfl(kbdim)
REAL(dp):: pcpcu(kbdim,klev)
INTEGER :: kcbot(kbdim),            kctop(kbdim),                      &
           kdtop(kbdim),            ktype(kbdim)
LOGICAL :: lddraf(kbdim),           ldcum(kbdim)
REAL(dp):: pxtenh(kbdim,klev,ktrac),                                   &
           pmfuxt(kbdim,klev,ktrac),pmfdxt(kbdim,klev,ktrac)
!
INTEGER :: jl, jk, jt, ikb
REAL(dp):: zcons1, zcons2, zcucov, ztmelp2, zzp, zfac, zsnmlt          &
         , zrfl, zrnew, zrmin, zrfln, zdrfl, zrsum, zdpevap
REAL(dp):: zpsubcl(kbdim)
!---Included for scavenging in xtwetdep (Philip Stier, 28/03/01):-------
!
INTEGER, INTENT(IN) :: krow

REAL(dp):: pxtte(kbdim,klev,ktrac), pxtu(kbdim,klev,ktrac),            &
           ptu(kbdim,klev)

REAL(dp):: zmlwc(kbdim,klev),       zmiwc(kbdim,klev),                 &
           zmratepr(kbdim,klev),    zmrateps(kbdim,klev),              &
           zfrain(kbdim,klev),      zfsnow(kbdim,klev),                &
           zdpg(kbdim,klev),        zfevapr(kbdim,klev),               &
           zfsubls(kbdim,klev),     zaclc(kbdim,klev),                 &
           zmsnowacl(kbdim,klev),   zrhou(kbdim,klev)

REAL(dp):: zxtp1c(kbdim,klev,ktrac),zxtp10(kbdim,klev,ktrac),          &
           zdummy(kbdim,klev,ktrac)

REAL(dp):: zlfrac(kbdim,klev),                                         &
           pmwc(kbdim,klev),        pmrateprecip(kbdim,klev)

LOGICAL,PARAMETER :: lstrat=.FALSE. ! Flag indicating if xt_wetdep is 
                                    ! called from the stratiform scheme.

REAL(dp):: zepsscav,                zrgas,                            &
           zqtp1,                   zeps,                             &
           zso4l,                   zso2l,                            &
           ztmst

REAL(dp):: ze2,         ze3,        zza,        zzb,                  &
           zzq,         zzp2,       zhp,        zqhp,                 &
           ztc,         zzfac,      zp_so2,     zheneff

REAL(dp):: zcaa, &   !(Constants for partitioning of cloud water 
           zcab      ! into liquid and solid part)

REAL(dp):: zalpha    ! Fraction of cloud water in liquid phase
                        ! =>  (1-zalpha)      "   in solid  phase

!---End Included for scavenging-----------------------------------------
!
!*             SPECIFY CONSTANTS
!

  zcons1=cpd/(alf*g*time_step_len)
  zcons2=1._dp/(g*time_step_len)
  zcucov=0.05_dp
  ztmelp2=tmelt+2._dp
!
!---Included for scavenging in xtwetdep (Philip Stier, 28/03/01):-------
!
  ztmst=time_step_len
  zeps =EPSILON(1.0_dp)

  zcaa = 0.0059_dp   ! (Constants for partitioning of cloud water 
  zcab = 0.003102_dp !  into liquid and solid part)

  zmlwc(1:kproma,:)     = 0._dp
  zmiwc(1:kproma,:)     = 0._dp
  zlfrac(1:kproma,:)    = 1._dp
  zmratepr(1:kproma,:)  = 0._dp
  zmrateps(1:kproma,:)  = 0._dp
  zfrain(1:kproma,:)    = 0._dp
  zfsnow(1:kproma,:)    = 0._dp
  zdpg(1:kproma,:)      = 0._dp
  zfevapr(1:kproma,:)   = 0._dp
  zfsubls(1:kproma,:)   = 0._dp
  zmsnowacl(1:kproma,:) = 0._dp
  zxtp1c(1:kproma,:,:)  = 0._dp
  zxtp10(1:kproma,:,:)  = 0._dp

  zdummy(1:kproma,:,:)  = 0._dp

  zepsscav       = 1._dp
  zrgas          = 8.2E-02_dp

  !--- Set cloud cover to 1 below cloud top and bottom:

  DO jk=1, klev
     DO jl=1, kproma
        IF (jk>=kctop(jl) .AND. jk<=kcbot(jl)) THEN
           zaclc(jl,jk) = 1._dp
        ELSE
           zaclc(jl,jk) = 0._dp
        END IF
     END DO
  END DO

!
!---End Included for scavenging-----------------------------------------
!
!
!
!*    1.0          DETERMINE FINAL CONVECTIVE FLUXES
!                  ---------------------------------
!
100 CONTINUE
!  itop=klev
  DO 110 jl=1,kproma
!     itop=MIN(itop,kctop(jl))
     IF(.NOT.ldcum(jl).OR.kdtop(jl).LT.kctop(jl)) lddraf(jl)=.FALSE.
     IF(.NOT.ldcum(jl)) ktype(jl)=0
110 END DO
  ktopm2=1
  DO 120 jk=ktopm2,klev
!DIR$ IVDEP
!OCL NOVREC
     DO 115 jl=1,kproma
        IF(ldcum(jl).AND.jk.GE.kctop(jl)-1) THEN
           pmfus(jl,jk)=pmfus(jl,jk)-pmfu(jl,jk)*                      &
                               (pcpcu(jl,jk)*ptenh(jl,jk)+pgeoh(jl,jk))
           pmfuq(jl,jk)=pmfuq(jl,jk)-pmfu(jl,jk)*pqenh(jl,jk)
           IF(lddraf(jl).AND.jk.GE.kdtop(jl)) THEN
              pmfds(jl,jk)=pmfds(jl,jk)-pmfd(jl,jk)*                   &
                               (pcpcu(jl,jk)*ptenh(jl,jk)+pgeoh(jl,jk))
              pmfdq(jl,jk)=pmfdq(jl,jk)-pmfd(jl,jk)*pqenh(jl,jk)
           ELSE
              pmfd(jl,jk)=0._dp
              pmfds(jl,jk)=0._dp
              pmfdq(jl,jk)=0._dp
              pdmfdp(jl,jk-1)=0._dp
           END IF
        END IF
115  END DO
!
     DO 1154 jt=1,ktrac
        DO 1152 jl=1,kproma
           IF(ldcum(jl).AND.jk.GE.kctop(jl)-1) THEN
              pmfuxt(jl,jk,jt)=pmfuxt(jl,jk,jt)                        &
                                     -pmfu(jl,jk)*pxtenh(jl,jk,jt)
              IF(lddraf(jl).AND.jk.GE.kdtop(jl)) THEN
                 pmfdxt(jl,jk,jt)=pmfdxt(jl,jk,jt)                     &
                                     -pmfd(jl,jk)*pxtenh(jl,jk,jt)
              ELSE
                 pmfdxt(jl,jk,jt)=0._dp
              ENDIF
           ELSE
              pmfuxt(jl,jk,jt)=0._dp
              pmfdxt(jl,jk,jt)=0._dp
           ENDIF
1152    END DO
1154 END DO
!
120 END DO
  DO 130 jk=ktopm2,klev
!DIR$ IVDEP
!OCL NOVREC
     DO 125 jl=1,kproma
        IF(ldcum(jl).AND.jk.GT.kcbot(jl)) THEN
           ikb=kcbot(jl)
           zzp=((paphp1(jl,klevp1)-paphp1(jl,jk))/                     &
                         (paphp1(jl,klevp1)-paphp1(jl,ikb)))
           zzp=MERGE(zzp**2,zzp,ktype(jl).EQ.3)
           pmfu(jl,jk)=pmfu(jl,ikb)*zzp
           pmfus(jl,jk)=pmfus(jl,ikb)*zzp
           pmfuq(jl,jk)=pmfuq(jl,ikb)*zzp
           pmful(jl,jk)=pmful(jl,ikb)*zzp
        END IF
125  END DO
!
     DO 1254 jt=1,ktrac
!DIR$ IVDEP
!OCL NOVREC
        DO 1252 jl=1,kproma
           IF(ldcum(jl).AND.jk.GT.kcbot(jl)) THEN
              ikb=kcbot(jl)
              zzp=(paphp1(jl,klevp1)-paphp1(jl,jk))/                   &
                            (paphp1(jl,klevp1)-paphp1(jl,ikb))
              zzp=MERGE(zzp**2,zzp,ktype(jl).EQ.3)
              pmfuxt(jl,jk,jt)=pmfuxt(jl,ikb,jt)*zzp
           ENDIF
1252    END DO
1254 END DO
!
130 END DO
!
!
!*    2.            CALCULATE RAIN/SNOW FALL RATES
!*                  CALCULATE MELTING OF SNOW
!*                  CALCULATE EVAPORATION OF PRECIP
!                   -------------------------------
!
200 CONTINUE
  DO 210 jl=1,kproma
     prfl(jl)=0._dp
     psfl(jl)=0._dp
     prain(jl)=0._dp
210 END DO
  DO 220 jk=ktopm2,klev
     DO 215 jl=1,kproma
!---Included for scavenging in xtwetdep (Philip Stier, 28/03/01):-------
        zdpg(jl,jk)=(paphp1(jl,jk+1)-paphp1(jl,jk))/g
!---End Included for scavenging-----------------------------------------
        IF(ldcum(jl)) THEN
           prain(jl)=prain(jl)+pdmfup(jl,jk)
!---Included for scavenging in xtwetdep (Philip Stier, 28/03/01):-------

           ztc=ptu(jl,jk)-tmelt
           zzfac=MERGE(1._dp,0._dp,ztc<0._dp)
           zalpha=(1._dp-zzfac)+zzfac*(zcaa+(1._dp-zcaa)*exp(-zcab*ztc**2))

           zmlwc(jl,jk)=zalpha*pmwc(jl,jk)
           zmiwc(jl,jk)=(1._dp-zalpha)*pmwc(jl,jk)

           zmratepr(jl,jk)=zalpha*pmrateprecip(jl,jk)
           zmrateps(jl,jk)=(1._dp-zalpha)*pmrateprecip(jl,jk)

!---End Included for scavenging-----------------------------------------
           IF(pten(jl,jk).GT.tmelt) THEN
              prfl(jl)=prfl(jl)+pdmfup(jl,jk)+pdmfdp(jl,jk)
              IF(psfl(jl).GT.0._dp.AND.pten(jl,jk).GT.ztmelp2) THEN
                 zfac=zcons1*(1._dp+vtmpc2*pqen(jl,jk))                &
                             *(paphp1(jl,jk+1)-paphp1(jl,jk))
                 zsnmlt=MIN(psfl(jl),zfac*(pten(jl,jk)-ztmelp2))
                 pdpmel(jl,jk)=zsnmlt
                 psfl(jl)=psfl(jl)-zsnmlt
                 prfl(jl)=prfl(jl)+zsnmlt
              END IF
           ELSE
              psfl(jl)=psfl(jl)+pdmfup(jl,jk)+pdmfdp(jl,jk)
           END IF
        END IF
!---Included for scavenging in xtwetdep (Philip Stier, 28/03/01):-------
        zfrain(jl,jk)=prfl(jl)
        zfsnow(jl,jk)=psfl(jl)
!---End Included for scavenging-----------------------------------------      
215  END DO
220 END DO
  DO 230 jl=1,kproma
     prfl(jl)=MAX(prfl(jl),0._dp)
     psfl(jl)=MAX(psfl(jl),0._dp)
     zpsubcl(jl)=prfl(jl)+psfl(jl)
230 END DO
  DO 240 jk=ktopm2,klev
     DO 235 jl=1,kproma
       IF(ldcum(jl).AND.jk.GE.kcbot(jl).AND.zpsubcl(jl).GT.1.e-20_dp)  &
                                                                  THEN
           zrfl=zpsubcl(jl)
           zrnew=(MAX(0._dp,SQRT(zrfl/zcucov)-                         &
                        cevapcu(jk)*(paphp1(jl,jk+1)-paphp1(jl,jk))*   &
                        MAX(0._dp,pqsen(jl,jk)-pqen(jl,jk))))**2*zcucov
           zrmin=zrfl-zcucov*MAX(0._dp,0.8_dp*pqsen(jl,jk)-pqen(jl,jk))&
                        *zcons2*(paphp1(jl,jk+1)-paphp1(jl,jk))
           zrnew=MAX(zrnew,zrmin)
           zrfln=MAX(zrnew,0._dp)
           zdrfl=MIN(0._dp,zrfln-zrfl)
           pdmfup(jl,jk)=pdmfup(jl,jk)+zdrfl
!---Included for scavenging in xtwetdep (Philip Stier, 28/03/01):-------
           zdpevap=-zdrfl
           zfevapr(jl,jk)=zdpevap*prfl(jl)/zpsubcl(jl)
           zfsubls(jl,jk)=zdpevap*psfl(jl)/zpsubcl(jl)
!---End Included for scavenging-----------------------------------------  
           zpsubcl(jl)=zrfln
       END IF
235  END DO
240 END DO

!---Included for scavenging in xtwetdep (Philip Stier, 28/03/01):-------
!
!*    3.0          SCAVENGING IN CONVECTIVE CLOUDS
!                  -------------------------------

 anywetdep: IF (trlist%anywetdep/=0) THEN

    DO 7204 jk=ktopm2,klev
       DO 7202 jl=1,kproma            

          !--- Calculate the solubility of SO2. Total sulfate
          !    is only used to calculate the ph of cloud water:
                
          zqtp1=1._dp/ptu(jl,jk)-1._dp/298._dp
          ze2=1.23_dp*EXP(3020._dp*zqtp1)
          ze3=1.2e-02_dp*EXP(2010._dp*zqtp1)
!ham_ps: should be moist density!
          zrhou(jl,jk)=(paphp1(jl,jk+1)+paphp1(jl,jk))*0.5_dp/(ptu(jl,jk)*rd)

          IF (zmlwc(jl,jk) > 1.E-15_dp) THEN
             zfac=1000._dp/(zmlwc(jl,jk)*32.064_dp)
          ELSE
             zfac=0._dp
          END IF

          zso4l=(pxtu(jl,jk,idt_so4)   + pxtu(jl,jk,idt_ms4ns) + &
                 pxtu(jl,jk,idt_ms4ks) + pxtu(jl,jk,idt_ms4as) + &
                 pxtu(jl,jk,idt_ms4cs)                       )*zfac
          zso4l=MAX(zso4l,0._dp)
          zso2l=pxtu(jl,jk,idt_so2)*zfac
          zso2l=MAX(zso2l,0._dp)
          zza=ze2*8.2e-02_dp*ptu(jl,jk)*zmlwc(jl,jk)*zrhou(jl,jk)*1.e-03_dp
          zzb=2.5e-06_dp+zso4l
          zzp=(zza*ze3-zzb-zza*zzb)/(1._dp+zza)
          zzq=-zza*ze3*(zzb+zso2l)/(1._dp+zza)
          zzp=0.5_dp*zzp
          zzp2=zzp*zzp
          zhp=-zzp+SQRT(zzp2-zzq)
          zqhp=1._dp/zhp
          zheneff=1._dp+ze3*zqhp
           
          zp_so2=zza*zheneff

          !--- zlfrac denotes the liquid fraction of the total SO2:

          zlfrac(jl,jk)=zp_so2/(1._dp+zp_so2)

          !--- set cloud cover to zero for non precipitating clouds:

          IF(ktype(jl)==0 .OR. (zfrain(jl,jk)+zfsnow(jl,jk))<zeps) THEN
             zaclc(jl,jk)=0._dp
          ENDIF

7202   END DO
7204 END DO

    CALL xt_wetdep(kbdim,  kproma,    klev,     ktopm2,   & ! dimensions
                   krow,                                  & ! longitude (kproma-block) index
                   pxtenh, pxtu,                          & ! tracers
                   pmfu,   pmfuxt,    zrhou,              & ! convective fluxes and corresp. mmr
                   zdummy,                                & ! stratiform properties (not needed for convective)
                   zmlwc,  zmiwc,     zmratepr, zmrateps, & ! cloud properties
                   zfrain, zfsnow,    zfevapr,  zfsubls,  & !   "       "
                   zaclc,  zmsnowacl, lstrat,             & !   "       "
                   ptu,    zdpg,      zlfrac,             & ! thermodynamic quantities
                   pxtte                                  ) ! fraction of SO2 in the liquid phase

 END IF anywetdep

!---End Included for scavenging-----------------------------------------  

  DO 250 jl=1,kproma
     zrsum=prfl(jl)+psfl(jl)
     zdpevap=zpsubcl(jl)-zrsum
     prfl(jl)=prfl(jl)+zdpevap*prfl(jl)*(1._dp/MAX(1.e-20_dp,zrsum))
     psfl(jl)=psfl(jl)+zdpevap*psfl(jl)*(1._dp/MAX(1.e-20_dp,zrsum))
250 END DO
!
  RETURN
END SUBROUTINE cuflx
