SUBROUTINE xt_aero_prop(kproma, kbdim, klev, krow, &  ! ECHAM indices
                        pxtm1,         pxtte       ) !   "   tracers

  USE mo_kind,         ONLY: dp
  USE mo_time_control, ONLY: time_step_len
  USE mo_tracer,       ONLY: AEROSOLNUMBER, AEROSOLMASS, ntrac, &
                             trlist,   flag
  USE mo_aero,         ONLY: cmode
  USE mo_aero_m7,      ONLY: ram2cmr, nmod
  USE mo_constants,    ONLY: api
  USE mo_aero_mem,     ONLY: rwet, densaer


  IMPLICIT NONE

  !--- Parameters:

  INTEGER, INTENT(IN)    :: kproma, kbdim, klev, krow
  REAL(dp),INTENT(IN)    :: pxtm1(kbdim,klev,ntrac)

  REAL(dp),INTENT(INOUT) :: pxtte(kbdim,klev,ntrac)

  !--- Streams and stream elements:

  REAL(dp),      POINTER :: rwet_p(:,:,:)
  REAL(dp),      POINTER :: densaer_p(:,:,:)

  !--- Local Variables:

  INTEGER                :: imod, jt, jmod

  REAL(dp)               :: zeps

  REAL(dp)               :: zmasssum(kbdim,klev,nmod),  zdenssum(kbdim,klev,nmod)

  REAL(dp)               :: zxtp1(kbdim,klev,ntrac)


  !--- 0) Initializations:

  zeps=EPSILON(1._dp)

  zmasssum(:,:,:)=0._dp
  zdenssum(:,:,:)=0._dp

  !--- Calculate tracer mixing ratios at t=t+1
  !    Convert sulfuric species from kg(S)/kg(air) to kg(tracer)/kg(air):

  DO jt=1, ntrac

     SELECT CASE (trlist%ti(jt)%basename)
     CASE('SO2')
        zxtp1(1:kproma,:,jt)=pxtm1(1:kproma,:,jt)+pxtte(1:kproma,:,jt)*time_step_len * 2.0_dp   ! * M(SO2)/M(S)=64/32
     CASE('SO4')
        zxtp1(1:kproma,:,jt)=pxtm1(1:kproma,:,jt)+pxtte(1:kproma,:,jt)*time_step_len * 3.06_dp  ! * M(H2SO4)/M(S
     CASE DEFAULT
        zxtp1(1:kproma,:,jt)=pxtm1(1:kproma,:,jt)+pxtte(1:kproma,:,jt)*time_step_len
     END SELECT

  END DO

  !--- 1) Calculate accumulated properties:

  DO  jt=1, ntrac

     IF (trlist%ti(jt)%nphase==AEROSOLMASS) THEN

        imod=trlist%ti(jt)%mode

        !--- Sum mass mixing ratio for each mode:

        zmasssum(1:kproma,:,imod)=zmasssum(1:kproma,:,imod)+zxtp1(1:kproma,:,jt)

        !--- Average density of each compound weighted with the mass mixing ratio:

        zdenssum(1:kproma,:,imod)=zdenssum(1:kproma,:,imod)+zxtp1(1:kproma,:,jt)*flag('density',jt)

     END IF

  END DO

  !--- 2) Compute average density for each mode:

  DO jmod=1, nmod

     densaer_p => densaer(jmod)%ptr

     WHERE(zmasssum(1:kproma,:,jmod) > zeps) 

        densaer_p(1:kproma,:,krow)=zdenssum(1:kproma,:,jmod)/zmasssum(1:kproma,:,jmod)

     ELSEWHERE

        densaer_p(1:kproma,:,krow)=0._dp

     END WHERE

  END DO

  !--- 3) Computation of the number median diameter of the modes and store in stream:

  DO jt=1, ntrac
     IF(trlist%ti(jt)%nphase==AEROSOLNUMBER) THEN

        imod=trlist%ti(jt)%mode

        rwet_p    => rwet(imod)%ptr
        densaer_p => densaer(imod)%ptr

        WHERE (zxtp1(1:kproma,:,jt) > 1.E-30_dp .AND. densaer_p(1:kproma,:,krow) > 1.E-30_dp)

           rwet_p(1:kproma,:,krow)=((3._dp/(4._dp*api))*(zmasssum(1:kproma,:,imod)/zxtp1(1:kproma,:,jt))/ &
                                       densaer_p(1:kproma,:,krow)                      )**(1._dp/3._dp) * &
                                       ram2cmr(imod)

        ELSEWHERE

           rwet_p(1:kproma,:,krow)=0._dp

        END WHERE

     END IF
  END DO

END SUBROUTINE xt_aero_prop
